/*------------------------------------------------------------------------------
 bin/djv_convert/app_cmdln.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include "etc.h"
#include <djv_image/io.h>

namespace djv_convert {

void App::cmdln(Str & in) throw (Error)
{
  DJV_DBG_OBJ("cmdln");
  DJV_DBG_PRN("in = " << in);
  
  djv_image::app::App::cmdln(in);
  if (exit() != djv_base::app::EXIT_DEFAULT) return;

  if (in.empty())
  {
    exit(djv_base::app::EXIT_HELP);
    return;
  }
  
  List<Str> arg_list;
  Str arg;
try
{
  while (! in.empty())
  {
    in >> arg;
    
    // Options.

    if ("-mirror" == arg) in >> _image.mirror;
    else if ("-scale" == arg) in >> _image.scale;
    else if ("-resize" == arg) in >> _image.size;
    else if ("-width" == arg) in >> _image.size.x;
    else if ("-height" == arg) in >> _image.size.y;
    else if ("-channel" == arg) in >> _image.channel;
    
    // Input.

    else if ("-layer" == arg) in >> _input.layer;
    else if ("-proxy" == arg) in >> _input.proxy;
    else if ("-time" == arg)
    {
      in >> _input.start;
      in >> _input.end;
    }
    else if ("-timeout" == arg) in >> _input.timeout;
    
    // Output.
    
    else if ("-pixel" == arg)
    {
      djv_image::PIXEL value = djv_image::PIXEL(0);
      in >> value;
      _output.pixel = std::auto_ptr<djv_image::PIXEL>(
        new djv_image::PIXEL(value));
    }
    else if ("-speed" == arg)
    {
      djv_image::time::FPS value = djv_image::time::FPS(0);
      in >> value;
      _output.speed = std::auto_ptr<djv_image::time::FPS>(
        new djv_image::time::FPS(value));
    }
    else if ("-tag" == arg)
    {
      Str name, value;
      in >> name;
      in >> value;
      _output.tag[name] = value;
    }
    else if ("-x_tag" == arg) _output.tag_auto = false;

    // Arguments.
    
    else arg_list += arg;
  }
}
catch (Str) { throw Error(arg); }
  DJV_DBG_PRN("arg list = " << arg_list);
  
  if (! arg_list.size()) throw Error(error_cmdln_input);
  _in = arg_list.pop_front();
  if (! arg_list.size()) throw Error(error_cmdln_output);
  _out = arg_list.pop_front();
  if (! arg_list.empty()) throw Error(arg_list[0]);
  
  if (_in.seq_valid()) _in.type = file::SEQ;
  if (_out.seq_valid()) _out.type = file::SEQ;
}

Str App::cmdln_help() const
{
  return str::Format(label_cmdln_help).
    arg(str::label_bool(), ", ").
    arg(djv_image::gl_data::label_channel(), ", ").
    arg(djv_image::data::label_proxy(), ", ").
    arg(str::label(_input.proxy)).
    arg(_input.timeout).
    arg(djv_image::pixel::label_pixel(), ", ").
    arg(djv_image::time::label_fps(), ", ").
    arg(str::label_bool(), ", ").
    arg(str::label(_output.tag_auto)).
    arg(djv_image::app::App::cmdln_help());
}

const Str label_cmdln_help =
"\n"
"djv_convert\n"
"-----------\n"
"\n"
"Command line utility for image conversion.\n"
"\n"
"Usage\n"
"-----\n"
"\n"
"* djv_convert (input) (output) [option]...\n"
"\n"
"Options\n"
"-------\n"
"\n"
"Image:\n"
"\n"
"* -mirror (x) (y)\n"
"  Mirror: %%.\n"
"\n"
"* -scale (x) (y)\n"
"  Scale.\n"
"\n"
"* -resize (width) (height)\n"
"  Resize.\n"
"\n"
"* -width (value)\n"
"  Resize width, preserving aspect ratio.\n"
"\n"
"* -height (value)\n"
"  Resize height, preserving aspect ratio.\n"
"\n"
"* -channel (value)\n"
"  Channel: %%.\n"
"\n"
"Input:\n"
"\n"
"* -layer (value)\n"
"  Layer index.\n"
"\n"
"* -proxy (value)\n"
"  Proxy scale: %%: Value: %%.\n"
"\n"
"* -time (start) (end)\n"
"  Select start and end time.\n"
"\n"
"* -timeout (value)\n"
"  Maximum number of seconds to wait for each input frame. Value: %%.\n"
"\n"
"Output:\n"
"\n"
"* -pixel (value)\n"
"  Pixel type: %%.\n"
"\n"
"* -speed (value)\n"
"  Speed: %%.\n"
"\n"
"* -tag (name) (value)\n"
"  Set an image tag.\n"
"\n"
"* -tag_auto (value)\n"
"  Automatic image tag generation: %%. Value: %%.\n"
"\n"
"General Options\n"
"---------------\n"
"\n"
"%%"
"\n"
"Examples\n"
"--------\n"
"\n"
"Convert an image:\n"
"> djv_convert input.sgi output.tga\n"
"\n"
"Convert an image sequence:\n"
"> djv_convert input.1-100.sgi output.1.tga\n"
"(Only the first frame of the output is specified.)\n"
"\n"
"Create an RLE compressed image sequence:\n"
"> djv_convert input.1-100.sgi output.1.tga -io_save tga compress rle\n"
"\n"
"Convert an image sequence to a movie:\n"
"> djv_convert input.0001-0100.dpx output.mov\n"
"\n"
"Convert a movie to an image sequence:\n"
"> djv_convert input.mov output.1.tga\n"
"\n"
"Convert pixel type:\n"
"> djv_convert input.sgi output.sgi -pixel rgb u16\n"
"\n"
"Scale an image by half:\n"
"> djv_convert input.tga output.tga -scale 0.5 0.5\n"
"\n"
"Resize an image:\n"
"> djv_convert input.tga output.tga -resize 2048 1556\n"
"\n"
"Convert a Cineon file to a linear format using the default settings:\n"
"> djv_convert input.cin output.tga\n"
"\n"
"Convert a Cineon file to a linear format using custom print settings (black "
"point, white point, gamma, and soft clip):\n"
"> djv_convert input.cin output.tga -io_load cineon print 95 685 2.2 10\n"
"\n"
"Save video lookup table to a file:\n"
"> djv_convert 0.vlut output.lut\n"
"\n"
"Load video lookup table from a file:\n"
"> djv_convert input.lut 0.vlut\n"
"\n";

}

