/*------------------------------------------------------------------------------
 bin/djv_view/app.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"

namespace djv_view {

App::App(int & argc, char ** argv) :
  djv_ui::app::App("djv_view", argc, argv),
  _shortcut_prefs(new shortcut::Prefs),
  _file_cache(new image_cache::Cache),
  _file_prefs(new file::Prefs),
  _window_prefs(new window::Prefs),
  _view_prefs(new view::Prefs),
  _image_prefs(new image::Prefs),
  _playback_prefs(new playback::Prefs),
  _playback_global(new playback::Global)
{
  using namespace djv_base;
  
  DJV_DBG_OBJ("App::App");
  
  // Command line.
  
  try
  {
    Str tmp = cmdln_arg();
    cmdln(tmp);
  }
  catch (Error error)
  {
    this->error(djv_base::app::error_cmdln, error);
    djv_image::app::App::exit(djv_base::app::EXIT_ERROR);
  }
  if (djv_image::app::App::exit() != djv_base::app::EXIT_DEFAULT) return;
  
  valid(true);

  // Setup.
  
  _file_save = std::auto_ptr<file::Save>(new file::Save);
  
  _image_display.p = std::auto_ptr<image_display::Dialog>(
    new image_display::Dialog);
  _image_histogram.p = std::auto_ptr<image_histogram::Dialog>(
    new image_histogram::Dialog);
  _image_info.p = std::auto_ptr<image_info::Dialog>(
    new image_info::Dialog);
  _image_magnify.p = std::auto_ptr<image_magnify::Dialog>(
    new image_magnify::Dialog);
  _image_sample.p = std::auto_ptr<image_sample::Dialog>(
    new image_sample::Dialog);
  _window_factory = std::auto_ptr<window::Factory>(new window::Factory);
  
  djv_ui::prefs_dialog::global()->add_shortcut(shortcut::prefs_title,
    new shortcut::Prefs_Widget);
  djv_ui::prefs_dialog::global()->add(playback::prefs_title,
    new playback::Prefs_Widget);
  djv_ui::prefs_dialog::global()->add(image::prefs_title,
    new image::Prefs_Widget);
  djv_ui::prefs_dialog::global()->add(view::prefs_title,
    new view::Prefs_Widget);
  djv_ui::prefs_dialog::global()->add(window::prefs_title,
    new window::Prefs_Widget);
  djv_ui::prefs_dialog::global()->add(file::prefs_title,
    new file::Prefs_Widget);

  if (_input.size())
  {
    if (file::prefs()->cmdln_combine())
    {
      djv_base::File file(_input[0]);
      if (file.seq_valid())
        file.type = djv_base::file::SEQ;
      
      for (uint i = 1; i < _input.size(); ++i)
        file.seq_add(_input[i]);
      djv_base::file::seq_sort(&file.seq);

      _input.resize(1);
      _input[0] = file;
    }
    
    for (uint i = 0; i < _input.size(); ++i)
    {
      djv_base::File file(_input[i]);
      if (file.seq_valid())
        file.type = djv_base::file::SEQ;

      window::Widget * window = this->window();
      window->file_open(file);
      
      // XXX
      
      if (_cmdln.file_layer.get())
        window->file_layer(*_cmdln.file_layer);
      if (_cmdln.playback.get())
        window->playback(*_cmdln.playback);
      if (_cmdln.playback_frame.get())
        window->playback_frame(*_cmdln.playback_frame);
      if (_cmdln.playback_speed.get())
        window->playback_speed(*_cmdln.playback_speed);
      
      window->show();
    }
  }
  else
  {
    window()->show();
  }
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
}

shortcut::Prefs * App::shortcut_prefs() const
  { return _shortcut_prefs.get(); }
image_cache::Cache * App::file_cache() const
  { return _file_cache.get(); }
file::Save * App::file_save() const
  { return _file_save.get(); }
file::Prefs * App::file_prefs() const
  { return _file_prefs.get(); }
window::Factory * App::window_factory() const
  { return _window_factory.get(); }
window::Prefs * App::window_prefs() const
  { return _window_prefs.get(); }
view::Prefs * App::view_prefs() const
  { return _view_prefs.get(); }
image::Prefs * App::image_prefs() const
  { return _image_prefs.get(); }
image_display::Dialog * App::image_display() const
  { return _image_display.get(); }
image_histogram::Dialog * App::image_histogram() const
  { return _image_histogram.get(); }
image_info::Dialog * App::image_info() const
  { return _image_info.get(); }
image_magnify::Dialog * App::image_magnify() const
  { return _image_magnify.get(); }
image_sample::Dialog * App::image_sample() const
  { return _image_sample.get(); }
playback::Prefs * App::playback_prefs() const
  { return _playback_prefs.get(); }
playback::Global * App::playback_global() const
  { return _playback_global.get(); }

window::Widget * App::window() const
{
  window::Widget * out = window::factory()->add();
  
  if (_cmdln.file_layer.get())
    out->file_layer(*_cmdln.file_layer);
  if (_cmdln.file_proxy.get())
    out->file_proxy(*_cmdln.file_proxy);
  if (_cmdln.file_cache.get())
    out->file_cache(*_cmdln.file_cache);
  
  if (_cmdln.window_fullscreen.get())
    out->window_fullscreen(*_cmdln.window_fullscreen);
  if (_cmdln.window_toolbar.get())
    out->window_toolbar(*_cmdln.window_toolbar);
  if (_cmdln.window_playbar.get())
    out->window_playbar(*_cmdln.window_playbar);
  
  if (_cmdln.view_grid.get())
    out->view_grid(*_cmdln.view_grid);
  if (_cmdln.view_hud.get())
    out->view_hud(*_cmdln.view_hud);
  
  if (_cmdln.playback.get())
    out->playback(*_cmdln.playback);
  if (_cmdln.playback_frame.get())
    out->playback_frame(*_cmdln.playback_frame);
  if (_cmdln.playback_speed.get())
    out->playback_speed(*_cmdln.playback_speed);
  
  return out;
}

namespace shortcut {
Prefs * prefs() { return app()->shortcut_prefs(); }
}
namespace file {
Save * save() { return app()->file_save(); }
Prefs * prefs() { return app()->file_prefs(); }
}
namespace image_cache {
Cache * global() { return app()->file_cache(); }
}
namespace window {
Factory * factory() { return app()->window_factory(); }
Prefs * prefs() { return app()->window_prefs(); }
}
namespace view {
Prefs * prefs() { return app()->view_prefs(); }
}
namespace image {
Prefs * prefs() { return app()->image_prefs(); }
}
namespace image_display {
Dialog * dialog() { return app()->image_display(); }
}
namespace image_histogram {
Dialog * dialog() { return app()->image_histogram(); }
}
namespace image_info {
Dialog * dialog() { return app()->image_info(); }
}
namespace image_magnify {
Dialog * dialog() { return app()->image_magnify(); }
}
namespace image_sample {
Dialog * dialog() { return app()->image_sample(); }
}
namespace playback {
Prefs * prefs() { return app()->playback_prefs(); }
Global * global() { return app()->playback_global(); }
}

}

int main(int argc, char ** argv)
{
  int r = 1;
  try { r = djv_view::App(argc, argv).exec(); }
  catch (djv_base::Error in) { djv_base::error::print(in); }
  return r;
}

