/*------------------------------------------------------------------------------
 bin/djv_view/view.h
 
 View
 ----

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_VIEW_H
#define DJV_VIEW_VIEW_H

#include <djv_ui/image_view.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_image/image.h>
#include <djv_image/time.h>
#include <djv_base/file.h>

namespace djv_view {
namespace view {
using namespace djv_base;

//------------------------------------------------------------------------------
// Options

enum RESIZE
{
  RESIZE_NONE,
  RESIZE_FIT,
  RESIZE_CENTER
};

enum GRID
{
  GRID_NONE,
  GRID_1x1,
  GRID_10x10,
  GRID_100x100
};

enum HUD_OPTION
{
  HUD_NAME,
  HUD_SIZE,
  HUD_PROXY,
  HUD_PIXEL,
  HUD_TAG,
  HUD_FRAME,
  HUD_SPEED,
  _HUD_OPTION_SIZE
};

enum HUD_BACKGROUND
{
  HUD_BACKGROUND_NONE,
  HUD_BACKGROUND_SOLID,
  HUD_BACKGROUND_SHADOW
};

struct Hud
{
  Hud() :
    frame(0),
    speed_real(0.0),
    dropped_frames(false)
  {}

  djv_image::Info info;
  djv_image::Tag tag;
  int frame;
  djv_image::time::Speed speed;
  double speed_real;
  bool dropped_frames;
};

//------------------------------------------------------------------------------
// Widget

class Widget : public djv_ui::image_view::Widget
{
public:

  Widget();
  
  // View.
  
  void zoom(double);
  void zoom(double, const Vec2i & focus);
  void zoom_focus(double);
  double zoom() const;
  
  // Overlays.
  
  void grid(GRID);
  void grid_color(const djv_image::Color &);
  
  void hud(bool);
  void hud_info(const Hud &);
  void hud_option(const List<bool> &);
  void hud_background(HUD_BACKGROUND);
  void hud_color_foreground(const djv_image::Color &);
  void hud_color_background(const djv_image::Color &);
  
  // Signals.
  
  djv_ui::Signal<const Vec2i &> pick_sig;
  djv_ui::Signal<bool> scroll_sig;
  djv_ui::Signal<int> scroll_value_sig;
  djv_ui::Signal<const File &> file_sig;
  
  // Widget.
  
  int handle(int);
  void draw();
  const Vec2i & mouse() const;
  
  // Layout.

  const Vec2i & size_hint() const;
  const Vec2i & size_min() const;

private:

  void draw_grid();
  void draw_hud();
  static void draw_hud(
    const Str &,
    const Vec2i & position,
    HUD_BACKGROUND,
    const djv_image::Color & color_foreground,
    const djv_image::Color & color_background
  );
  
  Vec2i _view_tmp;
  double _zoom_tmp;
  GRID _grid;
  djv_image::Color _grid_color;
  bool _hud;
  Hud _hud_info;
  List<bool> _hud_option;
  HUD_BACKGROUND _hud_background;
  djv_image::Color _hud_color_foreground;
  djv_image::Color _hud_color_background;
  bool _inside;
  Vec2i _mouse, _mouse_td;
  Vec2i _size_min;
};

//------------------------------------------------------------------------------
// View Group

class Group : public djv_ui::Callback_Base
{
public:
  
  Group(
    djv_ui::menu::Widget *,
    djv_ui::layout::Horizontal * toolbar,
    const Group * copy = 0
  );
  ~Group();

  // Value.
  
  void zoom(double);
  void grid(GRID);
  void hud(bool);
  GRID grid() const;
  bool hud() const;
  
  // Widget.
  
  void menu_update(djv_ui::menu::Group *);
  void toolbar_update();
  
  // Signals.
  
  djv_ui::Signal<const Vec2i &> move_sig;
  djv_ui::Signal<bool> center_sig;
  djv_ui::Signal<double> zoom_sig;
  djv_ui::Signal<bool> zoom_reset_sig;
  djv_ui::Signal<bool> reset_sig, fit_sig;
  djv_ui::Signal<bool> overlay_sig;
  djv_ui::Signal<bool> update_sig;
  
private:

  DJV_CALLBACK(Group, zoom_in_callback, bool);
  DJV_CALLBACK(Group, zoom_out_callback, bool);
  DJV_CALLBACK(Group, zoom_inc_callback, bool);
  DJV_CALLBACK(Group, zoom_reset_callback, bool);
  DJV_CALLBACK(Group, zoom_dec_callback, bool);
  DJV_CALLBACK(Group, fit_callback, bool);
  DJV_CALLBACK(Group, grid_callback, GRID);
  DJV_CALLBACK(Group, hud_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Group, _left_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _right_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _up_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _down_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _center_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _zoom_in_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _zoom_out_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _zoom_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _reset_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _fit_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _grid_callback);
  DJV_FL_WIDGET_CALLBACK(Group, _hud_callback);

  GRID _grid;
  bool _hud;
  
  struct _Group;
  std::auto_ptr<_Group> _p;
};

//------------------------------------------------------------------------------
// Preferences

class Prefs : public djv_ui::Callback_Base
{
public:

  Prefs();
  ~Prefs();
  
  // Background.
  
  void background(const djv_image::Color &);
  const djv_image::Color & background() const;

  // Resizing.
  
  void resize(RESIZE);
  RESIZE resize() const;
  djv_ui::Signal<RESIZE> resize_sig;
  
  // Grid.
  
  void grid(GRID);
  void grid_color(const djv_image::Color &);
  GRID grid() const;
  const djv_image::Color & grid_color() const;
  djv_ui::Signal<GRID> grid_sig;

  // HUD.
  
  void hud(bool);
  void hud_option(HUD_OPTION, bool);
  void hud_background(HUD_BACKGROUND);
  void hud_color_foreground(const djv_image::Color &);
  void hud_color_background(const djv_image::Color &);
  bool hud() const;
  List<bool> hud_option() const;
  bool hud_option(HUD_OPTION in) const;
  HUD_BACKGROUND hud_background() const;
  const djv_image::Color & hud_color_foreground() const;
  const djv_image::Color & hud_color_background() const;
  djv_ui::Signal<bool> hud_sig;

private:

  djv_image::Color _background;
  RESIZE _resize;
  GRID _grid;
  djv_image::Color _grid_color;
  bool _hud;
  List<bool> _hud_option;
  HUD_BACKGROUND _hud_background;
  djv_image::Color _hud_color_foreground;
  djv_image::Color _hud_color_background;
};

//------------------------------------------------------------------------------
// Preferences Widget

class Prefs_Widget : public djv_ui::widget::Widget
{
public:

  Prefs_Widget();
  ~Prefs_Widget();

private:

  DJV_CALLBACK(Prefs_Widget, background_callback, const djv_image::Color &);
  DJV_CALLBACK(Prefs_Widget, resize_callback, int);
  DJV_CALLBACK(Prefs_Widget, grid_callback, int);
  DJV_CALLBACK(Prefs_Widget, grid_callback, const djv_image::Color &);
  DJV_CALLBACK(Prefs_Widget, hud_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_name_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_size_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_proxy_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_pixel_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_tag_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_frame_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_speed_callback, bool);
  DJV_CALLBACK(Prefs_Widget, hud_background_callback, int);
  DJV_CALLBACK(Prefs_Widget, hud_color_foreground_callback,
    const djv_image::Color &);
  DJV_CALLBACK(Prefs_Widget, hud_color_background_callback,
    const djv_image::Color &);
  
  struct _Prefs_Widget;
  std::auto_ptr<_Prefs_Widget> _p;
};

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, RESIZE &) throw (Str);
Str & operator >> (Str &, GRID &) throw (Str);
Str & operator >> (Str &, HUD_BACKGROUND &) throw (Str);
Str & operator << (Str &, RESIZE);
Str & operator << (Str &, GRID);
Str & operator << (Str &, HUD_BACKGROUND);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_resize();
const List<Str> & label_grid();
const List<Str> & label_hud_info();
const List<Str> & label_hud_mode();
const List<Str> & label_hud_background();

extern const Str
  menu_title,
  menu_left,
  menu_right,
  menu_up,
  menu_down,
  menu_center,
  menu_zoom_in,
  menu_zoom_out,
  menu_zoom_reset,
  menu_reset,
  menu_fit,
  menu_grid,
  menu_hud;

extern const Str
  toolbar_zoom,
  toolbar_zoom_format;

extern const Str
  tooltip_zoom,
  tooltip_zoom_in,
  tooltip_zoom_out,
  tooltip_zoom_reset,
  tooltip_fit;

extern const Str
  prefs_title,
  prefs_background_group,
  prefs_background,
  prefs_resize_group,
  prefs_resize,
  prefs_grid_group,
  prefs_grid,
  prefs_grid_color,
  prefs_hud_group,
  prefs_hud,
  prefs_hud_name,
  prefs_hud_size,
  prefs_hud_proxy,
  prefs_hud_pixel,
  prefs_hud_tag,
  prefs_hud_frame,
  prefs_hud_speed,
  prefs_hud_background,
  prefs_hud_color,
  prefs_hud_color_foreground,
  prefs_hud_color_background;

//------------------------------------------------------------------------------
// Global

Prefs * prefs();

}
}

#endif

