/*------------------------------------------------------------------------------
 include/djv_base/box.h
 
 Axis-Aligned Box
 ----------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_BOX_H
#define DJV_BOX_H

#include <djv_base/vec.h>

namespace djv_base {
namespace box {

//------------------------------------------------------------------------------
// Box Base

template<typename T, int D>
struct Box
{
  inline Box();
  inline Box(const Box<T, D> &);
  inline Box(const Vec<T, D> & position, const Vec<T, D> & size);
  inline Box(const Vec<T, D> & size);
  
  // Value.

  static const int dimension;
  Vec<T, D> position, size;
  
  inline void set(const Box<T, D> &);
  inline void position1(const Vec<T, D> &);
  inline Vec<T, D> position1() const;
  
  // Functions.
  
  inline void zero();
};

//------------------------------------------------------------------------------
// Two-Dimensional Box

template<typename T>
struct Box<T, 2>
{
  inline Box();
  inline Box(const Box<T, 2> &);
  inline Box(const Vec<T, 2> & position, const Vec<T, 2> & size);
  inline Box(const Vec<T, 2> & size);
  inline Box(T x, T y, T w, T h);
  inline Box(T w, T h);
  inline Box<T, 2> & operator = (const Box<T, 2> &);
  
  // Value.
  
  static const int dimension;
  Vec<T, 2> position, size;
  T & x, & y;
  T & w, & h;
  
  inline void set(const Box<T, 2> &);
  inline void position1(const Vec<T, 2> &);
  inline Vec<T, 2> position1() const;
  
  // Functions.
  
  inline void zero();
  
  inline operator Box<int, 2> () const;
  inline operator Box<double, 2> () const;
};

typedef Box<int, 2> Box2i;
typedef Box<double, 2> Box2f;

//------------------------------------------------------------------------------
// Three-Dimensional Box

template<typename T>
struct Box<T, 3>
{
  inline Box();
  inline Box(const Box<T, 3> &);
  inline Box(const Vec<T, 3> & position, const Vec<T, 3> & size);
  inline Box(const Vec<T, 3> & size);
  inline Box(T x, T y, T z, T w, T h, T d);
  inline Box(T w, T h, T d);
  inline Box<T, 3> & operator = (const Box<T, 3> &);

  // Value.
  
  static const int dimension;
  Vec<T, 3> position, size;
  T & x, & y, & z;
  T & w, & h, & d;
  
  inline void set(const Box<T, 3> &);
  inline void position1(const Vec<T, 3> &);
  inline Vec<T, 3> position1() const;
  
  // Functions.
  
  inline void zero();
  
  inline operator Box<int, 3> () const;
  inline operator Box<double, 3> () const;
};

typedef Box<int, 3> Box3i;
typedef Box<double, 3> Box3f;

//------------------------------------------------------------------------------
// Functions

// Construct a box from two corners.

template<typename T, int D>
inline Box<T, D> corners(const Vec<T, D> &, const Vec<T, D> &);

// Swap components.

template<typename T, int D>
inline Box<T, D> swap(const Box<T, D> &);

// Intersect.

template<typename T, int D>
inline Box<T, D> intersect(const Box<T, D> &, const Box<T, D> &);
template<typename T, int D>
inline bool intersect(const Box<T, D> &, const Vec<T, D> &);

// Bound.

template<typename T, int D>
inline Box<T, D> bound(const Box<T, D> &, const Box<T, D> &);

// Expand.

template<typename T, int D>
inline Box<T, D> expand(const Box<T, D> &, const Vec<T, D> &);

// Border.

template<typename T, int D>
inline Box<T, D> border(const Box<T, D> &, const Vec<T, D> &);

// Size greater than zero.

template<typename T, int D>
inline bool size_valid(const Box<T, D> &);

// Arithmetic.

template <typename T, int D>
inline Box<T, D> operator + (const Box<T, D> &, T);
template <typename T, int D>
inline Box<T, D> operator - (const Box<T, D> &, T);
template <typename T, int D>
inline Box<T, D> operator * (const Box<T, D> &, T);
template <typename T, int D>
inline Box<T, D> operator / (const Box<T, D> &, T);

template <typename T, int D>
inline Box<T, D> operator + (const Box<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Box<T, D> operator - (const Box<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Box<T, D> operator * (const Box<T, D> &, const Vec<T, D> &);
template <typename T, int D>
inline Box<T, D> operator / (const Box<T, D> &, const Vec<T, D> &);

//------------------------------------------------------------------------------
// Comparison

template <typename T, int D>
inline bool operator == (const Box<T, D> &, const Box<T, D> &);
template <typename T, int D>
inline bool operator != (const Box<T, D> &, const Box<T, D> &);

//------------------------------------------------------------------------------
// Conversion

template <typename T, typename T2, int D>
inline Box<T2, D> convert(const Box<T, D> &);

template<typename T, int D>
inline Str & operator >> (Str &, Box<T, D> &) throw (Str);
template<typename T, int D>
inline Str & operator << (Str &, const Box<T, D> &);

//------------------------------------------------------------------------------
// Debugging

template <typename T, int D>
inline Dbg & operator << (Dbg &, const Box<T, D> &);

}

//------------------------------------------------------------------------------
// Declarations

using box::Box;
using box::Box2i;
using box::Box2f;
using box::Box3i;
using box::Box3f;

}

#include <djv_base/box_.h>
#include <djv_base/box2_.h>
#include <djv_base/box3_.h>
#include <djv_base/box_etc_.h>

#endif

