/*------------------------------------------------------------------------------
 include/djv_base/box_etc_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv_base {
namespace box {

template<typename T, int D>
inline Box<T, D> corners(
  const Vec<T, D> & position,
  const Vec<T, D> & position1
) {
  const Vec<T, D> & a = vec::min(position, position1);
  const Vec<T, D> & b = vec::max(position, position1);

  Box<T, D> out;
  out.position = a;
  out.position1(b);
  return out;
}

template<typename T, int D>
inline Box<T, D> swap(const Box<T, D> & in)
{
  return Box<T, D>(vec::swap(in.position), vec::swap(in.size));
}

template<typename T, int D>
inline bool intersect(const Box<T, D> & a, const Vec<T, D> & b)
{
  const Vec<T, D> a1 = a.position1();

  for (int i = 0; i < a.dimension; ++i)
    if (! math::intersect(b.e[i], math::Range<T>(a.position.e[i], a1.e[i])))
      return false;
  return true;
}

template<typename T, int D>
inline Box<T, D> intersect(const Box<T, D> & a, const Box<T, D> & b)
{
  const Vec<T, D> a1 = a.position1();
  const Vec<T, D> b1 = b.position1();

  Box<T, D> out;
  Vec<T, D> position1;
  for (int i = 0; i < a.dimension; ++i)
  {
    math::Range<T> tmp;
    math::intersect(
      math::Range<T>(a.position.e[i], a1.e[i]),
      math::Range<T>(b.position.e[i], b1.e[i]),
      &tmp);
    out.position.e[i] = tmp.min;
    position1.e[i] = tmp.max;
  }
  out.position1(position1);
  return out;
}

template<typename T, int D>
inline Box<T, D> bound(const Box<T, D> & a, const Box<T, D> & b)
{
  const Vec<T, D> a1 = a.position1();
  const Vec<T, D> b1 = b.position1();
  
  Box<T, D> out;
  Vec<T, D> position1;
  for (int i = 0; i < a.dimension; ++i)
  {
    math::Range<T> tmp;
    math::bound(
      math::Range<T>(a.position.e[i], a1.e[i]),
      math::Range<T>(b.position.e[i], b1.e[i]),
      &tmp);
    out.position.e[i] = tmp.min;
    position1.e[i] = tmp.max;
  }
  out.position1(position1);
  return out;
}

template<typename T, int D>
inline Box<T, D> expand(const Box<T, D> & in, const Vec<T, D> & expand)
{
  return corners(
    vec::min(in.position, expand),
    vec::max(in.position1(), expand));
}

template<typename T, int D>
inline Box<T, D> border(const Box<T, D> & in, const Vec<T, D> & border)
{
  return Box<T, D>(in.position - border, in.size + border * T(2));
}

template<typename T, int D>
inline bool size_valid(const Box<T, D> & in)
{
  return vec::size_valid(in.size);
}

#define _BOX_FNC_OP(IN) \
  \
  template<typename T, int D> \
  inline Box<T, D> operator IN (const Box<T, D> & a, T b) \
  { \
    return Box<T, D>(a.position IN b, a.size IN b); \
  }

_BOX_FNC_OP(+)
_BOX_FNC_OP(-)
_BOX_FNC_OP(*)
_BOX_FNC_OP(/)

#define _BOX_FNC_OP2(IN) \
  \
  template<typename T, int D> \
  inline Box<T, D> operator IN (const Box<T, D> & a, const Vec<T, D> & b) \
  { \
    return Box<T, D>(a.position IN b, a.size IN b); \
  }

_BOX_FNC_OP2(+)
_BOX_FNC_OP2(-)
_BOX_FNC_OP2(*)
_BOX_FNC_OP2(/)

template<typename T, int D>
inline bool operator == (const Box<T, D> & a, const Box<T, D> & b)
{
  return a.position == b.position && a.size == b.size;
}

template<typename T, int D>
inline bool operator != (const Box<T, D> & a, const Box<T, D> & b)
{
  return ! (a == b);
}

template<typename T, typename T2, int D>
inline Box<T2, D> convert(const Box<T, D> & in)
{
  Box<T2, D> out;
  out.position = vec::convert<T, T2, D>(in.position);
  out.size = vec::convert<T, T2, D>(in.size);
  return out;
}

template<typename T, int D>
inline Str & operator << (Str & out, const Box<T, D> & in)
{
  return out << in.position << in.size;
}

template<typename T, int D>
inline Str & operator >> (Str & in, Box<T, D> & out) throw (Str)
{
  return in >> out.position >> out.size;
}

template<typename T, int D>
inline Dbg & operator << (Dbg & dbg, const Box<T, D> & in)
{
  return dbg << str::label(in);
}

}
}
