/*------------------------------------------------------------------------------
 include/djv_base/math_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace math {

template<class T>
inline T abs(T in)
{
  return in < T(0) ? -in : in;
}

template<class T>
inline T mod(T in, T mod)
{
  T tmp = in;
  if (mod != T(0))
  {
    tmp = in - (int(in / mod) * mod);
    if (tmp < T(0)) tmp += mod;
  }
  return tmp;
}

inline int wrap(int in, int min, int max)
{
  return max > min ? (min + mod(in - min, max - min + 1)) : min;
}

inline int step(int in, int v)
{
  return in <= v;
}

inline int pulse(int in, int min, int max)
{
  return step(min, in) - step(max, in);
}

template<class T>
inline T min(T a, T b)
{
  return a < b ? a : b;
}

template<class T>
inline T max(T a, T b)
{
  return a > b ? a : b;
}

template<class T>
inline T clamp(T in, T min, T max)
{
  return in < min ? min : (in > max ? max : in);
}

template<class T, class U>
inline T lerp(U in, T min, T max)
{
  return min + T(in * (max - min));
}

inline double smooth_step(double in, double min, double max)
{
  return lerp(in * in * (3.0 - (2.0 * in)), min, max);
}

template<typename T>
inline Range<T>::Range(T in) :
  min(in),
  max(in)
{}

template<typename T>
inline Range<T>::Range(T min, T max) :
  min(min),
  max(max)
{}

template<class T>
inline bool intersect(T in, const Range<T> & range)
{
  return in >= range.min && in < range.max;
}

inline bool intersect(int in, const Range<int> & range)
{
  return in >= range.min && in < range.max + 1;
}

template<class T>
inline bool intersect(const Range<T> & a, const Range<T> & b, Range<T> * out)
{
  out->min = math::max(a.min, b.min);
  out->max = math::min(a.max, b.max);
  return out->max > out->min;
}

inline bool intersect(const Range<int> & a, const Range<int> & b,
  Range<int> * out)
{
  out->min = math::max(a.min, b.min);
  out->max = math::min(a.max, b.max);
  return (out->max + 1) > out->min;
}

template<class T>
inline void bound(const Range<T> & a, const Range<T> & b, Range<T> * out)
{
  out->min = math::min(a.min, b.min);
  out->max = math::max(a.max, b.max);
}

template<typename T>
inline bool operator == (const Range<T> & a, const Range<T> & b)
{
  return a.min == b.min && a.max == b.max;
}

template<typename T>
inline bool operator != (const Range<T> & a, const Range<T> & b)
{
  return ! (a == b);
}

/*inline int ceil(double in)
{
  const int tmp = int(in);
  return tmp + (in > 0 && in != tmp);
}

inline int floor(double in)
{
  const int tmp = int(in);
  return tmp - (in < 0 && ! in != tmp);
}*/

inline double fraction(double in)
{
  return in - int(in);
}

inline int to_pow2(int in)
{
  int i = 0;
  for (; in > (1 << i); ++i) ;
  return 1 << i;
}

inline double degrees_to_radians(double in) { return in * 0.017453; }
inline double radians_to_degrees(double in) { return in * 57.29578; }

template<class T>
inline Dbg & operator << (Dbg & dbg, const Range<T> & in)
{
  return dbg << str::Format("%%-%%").arg(in.min).arg(in.max);
}

}
}
