/*------------------------------------------------------------------------------
 include/djv_base/mtx.h
 
 Matrices
 --------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_MTX_H
#define DJV_MTX_H

#include <djv_base/vec.h>
#include <djv_base/box.h>

namespace djv_base {
namespace mtx {

//------------------------------------------------------------------------------
// Matrix Base

template<typename T, int D>
struct Mtx
{
  inline Mtx();
  inline Mtx(const Mtx<T, D> &);
  inline Mtx(const T *);
  inline Mtx<T, D> & operator = (const Mtx<T, D> &);
  
  // Value.

  static const int dimension, dimension_x2;
  T e [D * D];
  
  inline void set(const Mtx<T, D> &);
  inline void set(const T *);
  
  // Functions.
  
  inline void zero();
  inline void identity();
};

//------------------------------------------------------------------------------
// 3x3 Matrix

template<typename T>
struct Mtx<T, 3>
{
  inline Mtx();
  inline Mtx(const Mtx<T, 3> &);
  inline Mtx(T, T, T, T, T, T, T, T, T);
  inline Mtx(const T *);
  inline Mtx<T, 3> & operator = (const Mtx<T, 3> &);

  // Value.
  
  static const int dimension, dimension_x2;
  T e [3 * 3];
  
  inline void set(const Mtx<T, 3> &);
  inline void set(const T *);
  inline void set(T, T, T, T, T, T, T, T, T);

  // Functions.
  
  inline void zero();
  inline void identity();
};

typedef Mtx<double, 3> Mtx3f;

//------------------------------------------------------------------------------
// 4x4 Matrix

template<typename T>
struct Mtx<T, 4>
{
  inline Mtx();
  inline Mtx(const Mtx<T, 4> &);
  inline Mtx(T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T);
  inline Mtx(const T *);
  inline Mtx<T, 4> & operator = (const Mtx<T, 4> &);

  // Value.
  
  static const int dimension, dimension_x2;
  T e [4 * 4];
  
  inline void set(const Mtx<T, 4> &);
  inline void set(const T *);
  inline void set(T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T);
  
  // Functions.
  
  inline void zero();
  inline void identity();
};

typedef Mtx<double, 4> Mtx4f;

//------------------------------------------------------------------------------
// Functions

// Transpose.

template<typename T, int D>
inline Mtx<T, D> transpose(const Mtx<T, D> &);

// Arithmetic.

template<typename T, int D>
inline Mtx<T, D> operator * (const Mtx<T, D> &, const Mtx<T, D> &);

// Translate.

template<typename T, int D, int D2>
inline Mtx<T, D> translate(const Vec<T, D2> &);
inline Mtx3f translate3f(const Vec2f &);

// Scale.

template<typename T, int D, int D2>
inline Mtx<T, D> scale(const Vec<T, D2> &);
inline Mtx3f scale3f(const Vec2f &);

// Rotate.

inline Mtx3f rotate3f(double);

// Transform.

template<typename T, int D, int D2>
inline Vec<T, D2> operator * (const Mtx<T, D> &, const Vec<T, D2> &);
template<typename T>
inline Box<T, 2> operator * (const Mtx<T, 3> &, const Box<T, 2> &);

//------------------------------------------------------------------------------
// Comparison

template<typename T, int D>
inline bool operator == (const Mtx<T, D> &, const Mtx<T, D> &);
template<typename T, int D>
inline bool operator != (const Mtx<T, D> &, const Mtx<T, D> &);

//------------------------------------------------------------------------------
// Conversion

template <typename T, typename T2, int D>
inline Mtx<T2, D> convert(const Mtx<T, D> &);

template <typename T>
Mtx<T, 4> mtx4(const Mtx<T, 3> &);

template<typename T, int D>
inline Str & operator >> (Str &, Mtx<T, D> &) throw (Str);
template<typename T, int D>
inline Str & operator << (Str &, const Mtx<T, D> &);

//------------------------------------------------------------------------------
// Debugging

template<typename T, int D>
inline Dbg & operator << (Dbg &, const Mtx<T, D> &);

}

//------------------------------------------------------------------------------
// Declarations

using mtx::Mtx;
using mtx::Mtx3f;
using mtx::Mtx4f;

}

#include <djv_base/mtx_.h>
#include <djv_base/mtx3_.h>
#include <djv_base/mtx4_.h>
#include <djv_base/mtx_etc_.h>

#endif

