/*------------------------------------------------------------------------------
 include/djv_image/io_cineon.h

 Image I/O - Cineon
 ------------------
 
 Kodak Cineon image file format.

 Cineon is a specialized file format for storing images scanned from
 motion-picture film.
 
 Print color space:
 * Black point: 0 - 1023. Value: 95.
 * White point: 0 - 1023. Value: 685.
 * Gamma: 0.01 - 4.0. Value: 1.7.
 * Soft clip: 0 - 50. Value: 0.

 Support:
 
 * Image data: RGB, 10-bit (the most common variety).
 * Interleaved channels only.

 References:
 
 * Kodak, "4.5 DRAFT - Image File Format Proposal for Digital Pictures"

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_CINEON_H
#define DJV_IMAGE_IO_CINEON_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_cineon {

//------------------------------------------------------------------------------
// Information

static const Str name = "Cineon";

static const List<Str> extension_list = List<Str>() << ".cin";

//------------------------------------------------------------------------------
// Image Tags

enum TAG
{
  TAG_SOURCE_OFFSET,
  TAG_SOURCE_FILE,
  TAG_SOURCE_TIME,
  TAG_SOURCE_INPUT_DEVICE,
  TAG_SOURCE_INPUT_MODEL,
  TAG_SOURCE_INPUT_SERIAL,
  TAG_SOURCE_INPUT_PITCH,
  TAG_SOURCE_GAMMA,
  TAG_FILM_FORMAT,
  TAG_FILM_FRAME,
  TAG_FILM_FRAME_RATE,
  TAG_FILM_FRAME_ID,
  TAG_FILM_SLATE
};

//------------------------------------------------------------------------------
// Color Space

enum COLOR
{
  COLOR_AUTO,
  COLOR_RAW,
  COLOR_PRINT
};

struct Print
{
  Print() :
    black(95),
    white(685),
    gamma(1.7)
  {}

  int black, white;
  double gamma;
};

struct Unprint
{
  Unprint() :
    black(95),
    white(685),
    gamma(1.7),
    soft_clip(0)
  {}

  int black, white;
  double gamma;
  int soft_clip;
};

//------------------------------------------------------------------------------
// Options

enum LOAD_OPTION
{
  LOAD_COLOR,
  LOAD_PRINT
};

struct Load_Option
{
  Load_Option() :
    color(COLOR_AUTO)
  {}

  COLOR color;
  Unprint print;
};

enum SAVE_OPTION
{
  SAVE_COLOR,
  SAVE_PRINT
};

struct Save_Option
{
  Save_Option() :
    color(COLOR_PRINT)
  {}

  COLOR color;
  Print print;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;

  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *, Tag *, time::Speed * = 0) throw (Error);

  Load_Option _option;
  bool _print;
  std::auto_ptr<Data> _print_lut;
  File _file;
  djv_base::Io _io;
  Image _image;
  Data _tmp;
  Cache _cache;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;

  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:

  void _open(const File &, const Tag &) throw (Error);

  Save_Option _option;
  File _file;
  djv_base::Io _io;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Functions

void info_load(
  djv_base::Io *,
  Info *,
  Tag *,
  time::Speed *,
  bool * print = 0
) throw (Error);

void info_save(
  djv_base::Io *,
  const Str & file,
  const Info &,
  const Tag &,
  COLOR
) throw (Error);

void info_update(djv_base::Io *) throw (Error);

void print(pixel::F32_T *, int size, const Print &);
void unprint(pixel::F32_T *, int size, const Unprint &);

Data print_lut(const Print &);
Data unprint_lut(const Unprint &);

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Print &, const Print &);
bool operator == (const Unprint &, const Unprint &);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, Print &) throw (Str);
Str & operator >> (Str &, Unprint &) throw (Str);
Str & operator >> (Str &, COLOR &) throw (Str);
Str & operator << (Str &, const Print &);
Str & operator << (Str &, const Unprint &);
Str & operator << (Str &, COLOR);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_color();
const List<Str> & label_tag();
const List<Str> & label_load_option();
const List<Str> & label_save_option();

}
}

#endif

