/*------------------------------------------------------------------------------
 include/djv_image/time.h
 
 Image Time
 ----------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_TIME_H
#define DJV_IMAGE_TIME_H

#include <djv_image/base.h>
#include <djv_base/file.h>

namespace djv_image {
namespace time {

//------------------------------------------------------------------------------
// Options

enum UNITS
{
  UNITS_TIMECODE,
  UNITS_FRAMES
};

//------------------------------------------------------------------------------
// Speed

enum FPS
{
  FPS_1,
  FPS_3,
  FPS_6,
  FPS_12,
  FPS_15,
  FPS_16,
  FPS_18,
  FPS_23_98,
  FPS_24,
  FPS_25,
  FPS_29_97,
  FPS_30,
  FPS_50,
  FPS_59_94,
  FPS_60,
  FPS_120
};

struct Speed
{
  Speed();
  Speed(int scale, int duration = 1);
  Speed(FPS);
  
  int scale, duration;

  void set(FPS);
  bool valid() const;
};

//------------------------------------------------------------------------------
// Time

struct Time : public file::Seq
{
  Time();
  Time(const List<int> &, const Speed & = Speed());
  Time(int start, int end, const Speed & = Speed());

  Speed speed;
};

//------------------------------------------------------------------------------
// Timecode

void timecode_to_time(
  uint32_t,
  int * hour,
  int * minute,
  int * second,
  int * frame
);

uint32_t time_to_timecode(
  int hour,
  int minute,
  int second,
  int frame
);

int timecode_to_frame(
  uint32_t timecode,
  const time::Speed &
);

uint32_t frame_to_timecode(
  int frame,
  const time::Speed &
);

Str timecode_to_str(uint32_t);

uint32_t str_to_timecode(const Str &);

//------------------------------------------------------------------------------
// Keycode

Str keycode_to_str(
  int id,
  int type,
  int prefix,
  int count,
  int offset
);

bool str_to_keycode(
  const Str &,
  int * id,
  int * type,
  int * prefix,
  int * count,
  int * offset
);

//------------------------------------------------------------------------------
// Comparison

bool operator == (const Speed &, const Speed &);
bool operator != (const Speed &, const Speed &);

bool operator == (const Time &, const Time &);
bool operator != (const Time &, const Time &);

//------------------------------------------------------------------------------
// Conversion

double speed_to_float(const Speed &);
Speed float_to_speed(double);

Str units_to_str(int frame, const Speed &);
int str_to_units(const Str &, const Speed &);

Str & operator >> (Str &, Speed &) throw (Str);
Str & operator >> (Str &, Time &) throw (Str);
Str & operator >> (Str &, UNITS &) throw (Str);
Str & operator >> (Str &, FPS &) throw (Str);
Str & operator << (Str &, const Speed &);
Str & operator << (Str &, const Time &);
Str & operator << (Str &, UNITS);
Str & operator << (Str &, FPS);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_units();
const List<Str> & label_fps();

//------------------------------------------------------------------------------
// Global

class Global
{
public:

  Global();
  ~Global();

  void speed(FPS);
  void units(UNITS);
  FPS speed() const;
  UNITS units() const;

private:

  FPS _speed;
  UNITS _units;
};

Global * global();

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, const Speed &);
Dbg & operator << (Dbg &, const Time &);

}

//------------------------------------------------------------------------------
// Declarations

using time::Time;

}

#endif

