/*------------------------------------------------------------------------------
 lib/djv_base/app.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_base/app.h>
#include <djv_base/file.h>
#include <djv_base/math.h>
#include <djv_base/sys.h>
#include <sstream>
#include <stdio.h>

namespace djv_base {
namespace app {

namespace {

App * _app = 0; // Global application pointer.

Str app_path(const Str & cmdln_name)
{
  Str out = File(cmdln_name).path;
  
  if (! out.empty())
  {
    out = file::path_absolute(out);
  }
  else
  {
    // Check search path and current directory.
    
    const List<Str> tmp = List<Str>() <<
      str::split(sys::env("PATH"), file::list_separator_list) <<
      ".";

    for (uint i = 0; i < tmp.size(); ++i)
    {
      const Str path = file::path_absolute(tmp[i]);
      const Str name = path + file::path_separator + cmdln_name;

      // XXX Check for Windows specific file extensions.
      
      if (
        File(name).stat() ||
        File(name + ".exe").stat() ||
        File(name + ".com").stat()
      ) {
        out = path;
        break;
      }
    }
  }
  
  return out;
}

}

App::App(const Str & name, int & argc, char ** argv) :
  _exit(EXIT_DEFAULT),
  _name(name),
  _endline(false),
  _separator(false)
{
  _app = this;
  
  DJV_DBG_OBJ("App::App");
  DJV_DBG_PRN("name = " << name);

  // Command line.
  
  if (argc) _cmdln_name = argv[0];
  
  for (int i = 1; i < argc; ++i)
    _cmdln_arg << Str(argv[i]);
  DJV_DBG_PRN("cmdln = " << _cmdln_arg);
  
  // Application path.
  
  _path = app_path(_cmdln_name);
  DJV_DBG_PRN("path = " << _path);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
}

int App::exec()
{
  switch (_exit)
  {
    case EXIT_HELP: message(cmdln_help()); break;
    case EXIT_INFO: message("\n" + info() + "\n"); break;
    case EXIT_ABOUT: message("\n" + about() + "\n"); break;
    default: break;
  }
  return _exit;
}

void App::exit(EXIT in)
{
  _exit = in;
}

EXIT App::exit() const { return _exit; }

const Str & App::name() const { return _name; }
const Str & App::path() const { return _path; }

Str App::info() const
{
  return str::Format(label_info).
    arg(DJV_VERSION_MAJOR DJV_VERSION_MINOR).
    arg(sys::search_path(), ", ");
}

Str App::about() const
{
  return str::Format(label_about).
    arg(DJV_VERSION_MAJOR DJV_VERSION_MINOR);
}

void App::message(const Str & in) const
{
  const List<Str> tmp = str::format(in, sys::terminal_width() - 2);
  for (uint i = 0; i < tmp.size(); ++i)
    print(" " + tmp[i] + " ");
}

void App::error(const Error & in, const Str & message) const
{
  if (_endline)
  {
    ::printf("\n");
    const_cast<App *>(this)->_endline = false;
  }
  
  const Str tmp = error::format(in);
  print(message.size() ? Str(str::Format(error_format).arg(tmp).arg(message)) :
    tmp);
}

void App::print(const Str & in, bool newline) const
{
  if (_separator)
  {
    const_cast<App *>(this)->_separator = false;
    ::printf("\n");
  }
  
  if (newline)
    ::printf("%s\n", in.c_str());
  else
  {
    ::printf("%s", in.c_str());
    ::fflush(stdout);
  }

  const_cast<App *>(this)->_endline = ! newline;
}

void App::separator() const
{
  const_cast<App *>(this)->_separator = true;
}

void App::cmdln(Str & in) throw (Error)
{
  DJV_DBG_OBJ("App::cmdln");
  DJV_DBG_PRN("in = " << in);
  
  List<Str> tmp;
  while (in.size())
  {
    Str arg;
    if (! str::serialize(&in, &arg)) throw Error(arg);
    
    if ("-help" == arg || "-h" == arg)
      exit(EXIT_HELP);
    else if ("-info" == arg)
      exit(EXIT_INFO);
    else if ("-about" == arg)
      exit(EXIT_ABOUT);
#if defined(DJV_MACOSX)
    else if ("-psn_" == Str(arg, 0, 5))
      ; // XXX Mac OS X process id.
#endif
    else
      tmp += str::serialize(arg);
  }
  in = str::join(tmp, ' ');
}

const Str & App::cmdln_name() const { return _cmdln_name; }
const Str & App::cmdln_arg() const { return _cmdln_arg; }

Str App::cmdln_help() const
{
  return str::Format(label_cmdln_help);
}

App * global() { return _app; }

}
}

