/*------------------------------------------------------------------------------
 lib/djv_base/math.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>
#if defined(WIN32)
#include <windows.h>
#else
#include <sys/time.h>
#endif
#include <math.h>
#include <stdlib.h>

namespace djv_base {
namespace math {

int pow(int in, int power)
{
  return floor(::pow(double(in), double(power)));
}

double pow(double in, double power)
{
  return ::pow(in, power);
}

double sqrt(double in)
{
  return ::sqrt(in);
}

double exp(double in)
{
  return ::exp(in);
}

double log(double in)
{
  return ::log(in);
}

double log_10(double in)
{
  return ::log10(in);
}

double anti_log_10(double in)
{
  return exp(in * log(10.0));
}

double bias(double in, double bias)
{
  return pow(in, log(bias) / log(0.5));
}

// XXX

namespace {

const double almost_zero = FLT_MIN * 2.0;
const double almost_one = 1.0 - almost_zero;

}

double gain(double in, double gain)
{
  if (in < almost_zero) return 0.0;
  else if (in > almost_one) return 1.0;
  const double tmp = log(1.0 - gain) / log(0.5);
  if (in < 0.5) return pow(2.0 * in, tmp) / 2.0;
  else return 1.0 - pow(2.0 * (1.0 - in), tmp) / 2.0;
}

double soft_clip(double in, double value)
{
  const double tmp = 1.0 - value;
  if (in > tmp) in = tmp + (1.0 - exp(-(in - tmp) / value)) * value;
  return in;
}

double sin(double in) { return ::sin(in); }
double cos(double in) { return ::cos(in); }
double tan(double in) { return ::tan(in); }

double arc_sin(double in) { return ::asin(in); }
double arc_cos(double in) { return ::acos(in); }
double arc_tan(double in) { return ::atan(in); }
double arc_tan(double a, double b) { return ::atan2(a, b); }

// XXX Zero?

double sec(double in)
{
  const double tmp = cos(in);
  return tmp != 0.0 ? (1.0 / tmp) : 0.0;
}

double co_sec(double in)
{
  const double tmp = sin(in);
  return tmp != 0.0 ? (1.0 / tmp) : 0.0;
}

double co_tan(double in)
{
  const double tmp = tan(in);
  return tmp != 0.0 ? (1.0 / tmp) : 0.0;
}

int ceil(double in)
{
  return int(::ceil(in));
}

int floor(double in)
{
  return int(::floor(in));
}

namespace {

double _rand()
{
  return ::rand() / double(RAND_MAX);
}

}

double rand() { return _rand(); }
double rand(double in) { return in * _rand(); }
double rand(double min, double max) { return min + (max - min) * _rand(); }

void rand_seed(uint in)
{
  if (! in)
  {
    // XXX Seed with current time.
    
#if defined(WIN32)
    in = ::GetTickCount();
#else
    struct timeval tmp = { 0, 0 };
    gettimeofday(&tmp, 0);
    in = tmp.tv_usec;
#endif
  }
  
  srand(in);
}

List<Range<int> > range(const List<int> & in)
{
  List<Range<int> > out;
  
  if (in.size()) out += Range<int>(in[0], in[0]);
  
  for (uint i = 1; i < in.size(); ++i)
    if (in[i] - 1 != out[out.size() - 1].max)
      out += Range<int>(in[i], in[i]);
    else
      out[out.size() - 1].max = in[i];

  return out;
}

}
}

