/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_fixed.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "gl_data_fixed.h"

namespace djv_image {
namespace gl_data {
namespace fixed {

namespace {

Info color_table_info(GLenum target, const Info & in)
{
  DJV_DBG_OBJ("color_table_info");
  DJV_DBG_PRN("in = " << in);

  Info out(in);

  GLint size = 0;
  const GLint format = gl::format(out.pixel, out.bgr);
  const GLint type = gl::type(out.pixel);
  do
  {
    const int pow2 = math::to_pow2(out.size.x);
    if (GLEW_ARB_imaging)
    {
      glColorTable(target, GL_RGBA, pow2, format, type, 0);
      glGetColorTableParameteriv(target, GL_COLOR_TABLE_WIDTH, &size);
    }
    else if (GL_SGI_color_table)
    {
      glColorTableSGI(target, GL_RGBA, pow2, format, type, 0);
      glGetColorTableParameterivSGI(target, GL_COLOR_TABLE_WIDTH_SGI, &size);
    }
    DJV_DBG_PRN("size = " << int(size));
    if (! size) out.size.x = out.size.x >> 2;
  }
  while (! size && out.size.x);

  return out;
}

}

void Fixed::draw(const djv_image::Data & in, const Option & option)
  throw (Error)
{
  DJV_DBG_OBJ("Fixed::draw");
  DJV_DBG_PRN("in = " << in);
  
  DJV_DBG_GL(glPushAttrib(
    GL_CURRENT_BIT |
    GL_ENABLE_BIT |
    GL_TRANSFORM_BIT |
    GL_PIXEL_MODE_BIT |
    GL_TEXTURE_BIT));

  DJV_DBG_GL(glDisable(GL_DITHER));

  // Profile.
  
  if (
    profile::LUT == option.profile.type &&
    vec::size_valid(option.profile.lut.size())
  ) {
    GLint max = 0;
    DJV_DBG_GL(glGetIntegerv(GL_MAX_PIXEL_MAP_TABLE, &max));
    const int size = math::min(
      math::to_pow2(option.profile.lut.size().x),
      int(max));
    DJV_DBG_PRN("pixel map max = " << int(max));
    DJV_DBG_PRN("profile lut = " << size);
    
    djv_image::Data tmp(Info(Vec2i(size, 1), pixel::RGBA_F32));
    Option _option;
    _option.xform.scale = Vec2f(option.profile.lut.size()) / Vec2f(tmp.size());
    gl_data::copy(option.profile.lut, &tmp, _option);

    mem::Buffer<GLfloat> data(size * 4);
    const pixel::F32_T * in_p = reinterpret_cast<pixel::F32_T *>(tmp.data());
    GLfloat * out_p = data();
    for (int i = 0; i < size; ++i, in_p += 4, out_p += 1)
    {
      DJV_DBG_PRN("lut[" << i << "] = " << in_p[0]);
      out_p[size * 0] = in_p[0];
      out_p[size * 1] = in_p[1];
      out_p[size * 2] = in_p[2];
      out_p[size * 3] = in_p[3];
    }

    DJV_DBG_GL(glPixelTransferi(GL_MAP_COLOR, GL_TRUE));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_R_TO_R, size, data() + size * 0));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_G_TO_G, size, data() + size * 1));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_B_TO_B, size, data() + size * 2));
    DJV_DBG_GL(glPixelMapfv(GL_PIXEL_MAP_A_TO_A, size, data() + size * 3));
  }
  
  // Channel.
  
  if (option.channel > 0)
  {
    DJV_DBG_PRN("channel = " << option.channel);
    
    bool mask [] = { GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE };
    mask[int(option.channel) - 1] = GL_TRUE;
    DJV_DBG_PRN("mask = " << mask[0] << " " << mask[1] << " " << mask[2] <<
      " " << mask[3]);
    DJV_DBG_GL(glColorMask(mask[0], mask[1], mask[2], mask[3]));
  }

  // Display.
  
  if (
    (GLEW_ARB_imaging || GL_SGI_color_table) &&
    vec::size_valid(option.display.lut.size())
  ) {
    GLenum target = 0;
    if (GLEW_ARB_imaging) target = GL_PROXY_COLOR_TABLE;
    else if (GL_SGI_color_table) target = GL_PROXY_COLOR_TABLE_SGI;
    const Info info(color_table_info(
      target, Info(Vec2i(option.display.lut.size().x, 1), pixel::RGBA_U8)));
    DJV_DBG_PRN("display lut = " << info);
    
    djv_image::Data tmp(info);
    Option _option;
    _option.xform.scale = Vec2f(option.display.lut.size()) / Vec2f(tmp.size());
    gl_data::copy(option.display.lut, &tmp, _option);

    state_unpack(info);
    if (GLEW_ARB_imaging)
    {
      target = GL_COLOR_TABLE;
      DJV_DBG_GL(glColorTable(
        target, GL_RGBA, info.size.x, GL_RGBA, GL_UNSIGNED_BYTE, tmp.data()));
    }
    else if (GL_SGI_color_table)
    {
      target = GL_COLOR_TABLE_SGI;
      DJV_DBG_GL(glColorTableSGI(
        target, GL_RGBA, info.size.x, GL_RGBA, GL_UNSIGNED_BYTE, tmp.data()));
    }
    DJV_DBG_GL(glEnable(target));
  }
  
  if (GLEW_ARB_imaging || GL_SGI_color_matrix)
  {
    DJV_DBG_PRN("color matrix");

    DJV_DBG_GL(glMatrixMode(GL_COLOR));
    DJV_DBG_GL(glLoadMatrixd(color_mtx(option.display.color).e));
    DJV_DBG_GL(glMatrixMode(GL_MODELVIEW));
  }

  if (
    (GLEW_ARB_imaging || GL_SGI_color_table) &&
    (
      option.display.levels != Display().levels ||
      option.display.soft_clip != Display().soft_clip
    )
  ) {
    GLenum target = 0;
    if (GLEW_ARB_imaging)
      target = GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE;
    else if (GL_SGI_color_table)
      target = GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI;

    const djv_image::Data lut = color_lut(
      option.display.levels, option.display.soft_clip);
    const Info info(
      color_table_info(target, Info(Vec2i(lut.size().x, 1), pixel::RGBA_U8)));
    DJV_DBG_PRN("color lut = " << info);

    djv_image::Data tmp(info);
    Option _option;
    _option.xform.scale = Vec2f(lut.size()) / Vec2f(tmp.size());
    gl_data::copy(lut, &tmp, _option);

    state_unpack(info);
    if (GLEW_ARB_imaging)
    {
      target = GL_POST_COLOR_MATRIX_COLOR_TABLE;
      DJV_DBG_GL(glColorTable(target, GL_RGBA, info.size.x, GL_RGBA,
        GL_UNSIGNED_BYTE, tmp.data()));
    }
    else if (GL_SGI_color_table)
    {
      target = GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI;
      DJV_DBG_GL(glColorTableSGI(target, GL_RGBA, info.size.x, GL_RGBA,
        GL_UNSIGNED_BYTE, tmp.data()));
    }
    DJV_DBG_GL(glEnable(target));
  }
  
  // Draw.
  
  const Info & info = in.info();
  const Vec2b mirror(
    info.mirror.x ? (! option.xform.mirror.x) : option.xform.mirror.x,
    info.mirror.y ? (! option.xform.mirror.y) : option.xform.mirror.y);
  const int proxy_scale = djv_image::data::proxy_scale(info.proxy);
  Vec2f scale = option.xform.scale * double(proxy_scale);
  Vec2f position = option.xform.position;
  if (mirror.x)
  {
    position.x += info.size.x * scale.x;
    scale.x = -scale.x;
  }
  if (mirror.y)
  {
    position.y += info.size.y * scale.y;
    scale.y = -scale.y;
  }
    
  state_unpack(info);
  DJV_DBG_GL(glRasterPos2i(0, 0));
  DJV_DBG_GL(glBitmap(0, 0, 0, 0, position.x, position.y, 0));
  DJV_DBG_GL(glPixelZoom(scale.x, scale.y));
  DJV_DBG_GL(glDrawPixels(
    info.size.x, info.size.y,
    gl::format(info.pixel, info.bgr),
    gl::type(info.pixel),
    in.data()));
  state_reset();

  DJV_DBG_GL(glPopAttrib());
}

}
}
}
