/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_info.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io_dpx_info.h"

namespace djv_image {
namespace io_dpx {
namespace info {

Header::Header()
{
  mem::set(0xff, &_file, sizeof(File));
  zero(_file.version, 8);
  zero(_file.name, 100);
  zero(_file.time, 24);
  zero(_file.creator, 100);
  zero(_file.project, 200);
  zero(_file.copyright, 200);

  mem::set(0xff, &_image, sizeof(Image));

  mem::set(0xff, &_source, sizeof(Source));
  zero(_source.file, 100);
  zero(_source.time, 24);
  zero(_source.input_device, 32);
  zero(_source.input_serial, 32);

  mem::set(0xff, &_film, sizeof(Film));
  zero(_film.id, 2);
  zero(_film.type, 2);
  zero(_film.offset, 2);
  zero(_film.prefix, 6);
  zero(_film.count, 4);
  zero(_film.format, 32);
  zero(_film.frame_id, 32);
  zero(_film.slate, 100);

  mem::set(0xff, &_tv, sizeof(Tv));
}

void Header::load(djv_base::Io * io, Info * info, Tag * tag,
  time::Speed * speed, bool * print) throw (Error)
{
  DJV_DBG_OBJ("Header::load");
  
  // Read.
  
  io->get(&_file, sizeof(File));
  
  DJV_DBG_PRN("magic = " << Str((char *)&_file.magic, 4));
  if (0 == mem::compare(&_file.magic, magic[0], 4))
    info->endian = mem::MSB;
  else if (0 == mem::compare(&_file.magic, magic[1], 4))
    info->endian = mem::LSB;
  else
    throw Error(name, io::error_unrecognized);

  io->get(&_image, sizeof(Image));
  io->get(&_source, sizeof(Source));
  io->get(&_film, sizeof(Film));
  io->get(&_tv, sizeof(Tv));

  if (info->endian != mem::endian())
  {
    DJV_DBG_PRN("endian");
    io->endian(true);
    endian();
  }
  
  // Information.

  if (_image.elem_size != 1)
    throw Error(name, io::error_unsupported);
  
  info->size = Vec2i(_image.size[0], _image.size[1]);
  DJV_DBG_PRN("size = " << info->size);
  
  switch (_image.orient)
  {
    case ORIENT_LEFT_RIGHT_TOP_BOTTOM: info->mirror.y = true; break;
    case ORIENT_RIGHT_LEFT_TOP_BOTTOM: info->mirror.x = true; break;
    case ORIENT_LEFT_RIGHT_BOTTOM_TOP: break;
    case ORIENT_RIGHT_LEFT_BOTTOM_TOP:
      info->mirror.x = info->mirror.y = true; break;
    case ORIENT_TOP_BOTTOM_LEFT_RIGHT:
    case ORIENT_TOP_BOTTOM_RIGHT_LEFT:
    case ORIENT_BOTTOM_TOP_LEFT_RIGHT:
    case ORIENT_BOTTOM_TOP_RIGHT_LEFT: break;
  }

  int pixel = -1;
  switch (_image.elem[0].packing)
  {
    case PACK:
    {
      int channels = 0;
      switch (_image.elem[0].descriptor)
      {
        case DESCRIPTOR_L: channels = 1; break;
        case DESCRIPTOR_RGB: channels = 3; break;
        case DESCRIPTOR_RGBA: channels = 4; break;
      }
      pixel = pixel::pixel_(channels, _image.elem[0].bit_depth, true);
    } break;
    case TYPE_A:
      switch (_image.elem[0].descriptor)
      {
        case DESCRIPTOR_RGB:
          switch (_image.elem[0].bit_depth)
          {
            case 10: pixel = pixel::RGB_U10; break;
          }
          break;
      }
      break;
  }
  if (-1 == pixel)
    throw Error(name, io::error_unsupported);
  info->pixel = PIXEL(pixel);
  DJV_DBG_PRN("pixel = " << info->pixel);
  
  switch (pixel::bit_depth(info->pixel))
  {
    case 8:
    case 10: info->align = 4; break;
  }

  if (_image.elem[0].encoding)
    throw Error(name, io::error_unsupported);
  
  if (valid(&_image.elem[0].line_padding) && _image.elem[0].line_padding)
    throw Error(name, io::error_unsupported);
  
  if (print)
    *print = TRANSFER_PRINT == _image.elem[0].transfer;
  
  // File image tags.
  
  const List<Str> & label = tag::label_tag();
  const List<Str> & label_dpx = io_dpx::label_tag();

  if (valid(_file.time, 24))
    (*tag)[label[tag::TIME]] = to_str(_file.time, 24);
  
  if (valid(_file.creator, 100))
    (*tag)[label[tag::CREATOR]] = to_str(_file.creator, 100);
  
  if (valid(_file.project, 200))
    (*tag)[label[tag::PROJECT]] = to_str(_file.project, 200);
  
  if (valid(_file.copyright, 200))
    (*tag)[label[tag::COPYRIGHT]] = to_str(_file.copyright, 200);
  
  // Source image tags.
  
  if (valid(&_source.offset[0]) && valid(&_source.offset[1]))
    (*tag)[label_dpx[TAG_SOURCE_OFFSET]] = str::join(List<Str>() <<
      str::int_to_str(_source.offset[0]) <<
      str::int_to_str(_source.offset[1]),
      " ");
  
  if (valid(&_source.center[0]) && valid(&_source.center[1]))
    (*tag)[label_dpx[TAG_SOURCE_CENTER]] = str::join(List<Str>() <<
      str::float_to_str(_source.center[0]) <<
      str::float_to_str(_source.center[1]),
      " ");
  
  if (valid(&_source.size[0]) && valid(&_source.size[1]))
    (*tag)[label_dpx[TAG_SOURCE_SIZE]] = str::join(List<Str>() <<
      str::int_to_str(_source.size[0]) <<
      str::int_to_str(_source.size[1]),
      " ");
  
  if (valid(_source.file, 100))
    (*tag)[label_dpx[TAG_SOURCE_FILE]] = to_str(_source.file, 100);
  
  if (valid(_source.time, 24))
    (*tag)[label[TAG_SOURCE_TIME]] = to_str(_source.time, 24);
  
  if (valid(_source.input_device, 32))
    (*tag)[label_dpx[TAG_SOURCE_INPUT_DEVICE]] =
      to_str(_source.input_device, 32);
  
  if (valid(_source.input_serial, 32))
    (*tag)[label_dpx[TAG_SOURCE_INPUT_SERIAL]] =
      to_str(_source.input_serial, 32);
  
  if (
    valid(&_source.border[0]) && valid(&_source.border[1]) &&
    valid(&_source.border[2]) && valid(&_source.border[3])
  )
    (*tag)[label_dpx[TAG_SOURCE_BORDER]] = str::join(List<Str>() <<
      str::int_to_str(_source.border[0]) <<
      str::int_to_str(_source.border[1]) <<
      str::int_to_str(_source.border[2]) <<
      str::int_to_str(_source.border[3]),
      " ");
  
  if (valid(&_source.pixel_aspect[0]) && valid(&_source.pixel_aspect[1]))
    (*tag)[label_dpx[TAG_SOURCE_PIXEL_ASPECT]] = str::join(List<Str>() <<
      str::int_to_str(_source.pixel_aspect[0]) <<
      str::int_to_str(_source.pixel_aspect[1]),
      " ");
  
  if (valid(&_source.scan_size[0]) && valid(&_source.scan_size[1]))
    (*tag)[label_dpx[TAG_SOURCE_SCAN_SIZE]] = str::join(List<Str>() <<
      str::float_to_str(_source.scan_size[0]) <<
      str::float_to_str(_source.scan_size[1]),
      " ");

  // Film image tags.
  
  if (
    valid(_film.id, 2) && valid(_film.type, 2) &&
    valid(_film.offset, 2) && valid(_film.prefix, 6) &&
    valid(_film.count, 4)
  )
    (*tag)[label[tag::KEYCODE]] = time::keycode_to_str(
      str::str_to_int(to_str(_film.id, 2)),
      str::str_to_int(to_str(_film.type, 2)),
      str::str_to_int(to_str(_film.prefix, 6)),
      str::str_to_int(to_str(_film.count, 4)),
      str::str_to_int(to_str(_film.offset, 2)));
  
  if (valid(_film.format, 32))
    (*tag)[label_dpx[TAG_FILM_FORMAT]] = to_str(_film.format, 32);
  
  if (valid(&_film.frame))
    (*tag)[label_dpx[TAG_FILM_FRAME]] = str::int_to_str(_film.frame);
  
  if (valid(&_film.sequence))
    (*tag)[label_dpx[TAG_FILM_SEQUENCE]] = str::int_to_str(_film.sequence);
  
  if (valid(&_film.hold))
    (*tag)[label_dpx[TAG_FILM_HOLD]]  = str::int_to_str(_film.hold);
  
  if (valid(&_film.frame_rate) && _film.frame_rate > speed_min)
  {
    if (speed) *speed = time::float_to_speed(_film.frame_rate);
    (*tag)[label_dpx[TAG_FILM_FRAME_RATE]] =
      str::float_to_str(_film.frame_rate);
  }
  
  if (valid(&_film.shutter))
    (*tag)[label_dpx[TAG_FILM_SHUTTER]] = str::float_to_str(_film.shutter);
  
  if (valid(_film.frame_id, 32))
    (*tag)[label_dpx[TAG_FILM_FRAME_ID]] = to_str(_film.frame_id, 32);
  
  if (valid(_film.slate, 100))
    (*tag)[label_dpx[TAG_FILM_SLATE]] = to_str(_film.slate, 100);

  // TV image tags.
    
  if (valid(&_tv.timecode))
    (*tag)[label[tag::TIMECODE]] = time::timecode_to_str(_tv.timecode);
  
  if (valid(&_tv.interlace))
    (*tag)[label_dpx[TAG_TV_INTERLACE]] = str::int_to_str(_tv.interlace);
  
  if (valid(&_tv.field))
    (*tag)[label_dpx[TAG_TV_FIELD]] = str::int_to_str(_tv.field);
  
  if (valid(&_tv.video_signal))
    (*tag)[label_dpx[TAG_TV_VIDEO_SIGNAL]] =
      str::int_to_str(_tv.video_signal);
  
  if (valid(&_tv.sample_rate[0]) && valid(&_tv.sample_rate[1]))
    (*tag)[label_dpx[TAG_TV_SAMPLE_RATE]] = str::join(List<Str>() <<
      str::float_to_str(_tv.sample_rate[0]) <<
      str::float_to_str(_tv.sample_rate[1]),
      " ");
  
  if (valid(&_tv.frame_rate) && _tv.frame_rate > speed_min)
  {
    if (speed) *speed = time::float_to_speed(_tv.frame_rate);
    (*tag)[label_dpx[TAG_TV_FRAME_RATE]] = str::float_to_str(_tv.frame_rate);
  }
  
  if (valid(&_tv.time_offset))
    (*tag)[label_dpx[TAG_TV_TIME_OFFSET]] = str::float_to_str(_tv.time_offset);
  
  if (valid(&_tv.gamma))
    (*tag)[label_dpx[TAG_TV_GAMMA]] = str::float_to_str(_tv.gamma);
  
  if (valid(&_tv.black_level))
    (*tag)[label_dpx[TAG_TV_BLACK_LEVEL]] = str::float_to_str(_tv.black_level);
  
  if (valid(&_tv.black_gain))
    (*tag)[label_dpx[TAG_TV_BLACK_GAIN]] = str::float_to_str(_tv.black_gain);
  
  if (valid(&_tv.breakpoint))
    (*tag)[label_dpx[TAG_TV_BREAK_POINT]] = str::float_to_str(_tv.breakpoint);
  
  if (valid(&_tv.white_level))
    (*tag)[label_dpx[TAG_TV_WHITE_LEVEL]] = str::float_to_str(_tv.white_level);
  
  if (valid(&_tv.integration_times))
    (*tag)[label_dpx[TAG_TV_INTEGRATION_TIMES]] =
      str::float_to_str(_tv.integration_times);

  // End.
  
  dbg();
  
  if (_file.image_offset)
    io->position(_file.image_offset);
}

void Header::save(djv_base::Io * io, const Str & file, const Info & info,
  const Tag & tag, ENDIAN endian, io_cineon::COLOR color) throw (Error)
{
  DJV_DBG_OBJ("Header::save");
  
  // Information.
  
  mem::copy("V2.0", _file.version, 4);
  
  _file.image_offset = 2048;
  _file.header_size = 2048 - 384;
  _file.industry_header_size = 384;
  _file.user_header_size = 0;
  _file.size = 0;
  _file.ditto_key = 1;
  
  _image.elem_size = 1;
  _image.size[0] = info.size.x;
  _image.size[1] = info.size.y;
  _image.orient = ORIENT_LEFT_RIGHT_TOP_BOTTOM;
  
  switch (info.pixel)
  {
    case pixel::L_U8:
    case pixel::L_U16:
    case pixel::L_F16: 
    case pixel::L_F32: _image.elem[0].descriptor = DESCRIPTOR_L; break;
    case pixel::RGB_U8:
    case pixel::RGB_U10:
    case pixel::RGB_U16:
    case pixel::RGB_F16: 
    case pixel::RGB_F32: _image.elem[0].descriptor = DESCRIPTOR_RGB; break;
    case pixel::RGBA_U8:
    case pixel::RGBA_U16:
    case pixel::RGBA_F16: 
    case pixel::RGBA_F32: _image.elem[0].descriptor = DESCRIPTOR_RGBA; break;
    default: break;
  }
  
  switch (info.pixel)
  {
    case pixel::RGB_U10: _image.elem[0].packing = TYPE_A; break;
    default: break;
  }
  
  const int bit_depth = pixel::bit_depth(info.pixel);  
  _image.elem[0].bit_depth = bit_depth;

  _image.elem[0].data_sign = 0;
  _image.elem[0].low_data = 0;
  switch (bit_depth)
  {
    case 8: _image.elem[0].high_data = 255; break;
    case 10: _image.elem[0].high_data = 1023; break;
    case 12: _image.elem[0].high_data = 4095; break;
    case 16: _image.elem[0].high_data = 65535; break;
  }
    
  if (io_cineon::COLOR_RAW == color)
  {
    _image.elem[0].transfer = TRANSFER_LINEAR;
    _image.elem[0].colorimetric = COLORIMETRIC_USER;
  }
  else
  {
    _image.elem[0].transfer = TRANSFER_PRINT;
    _image.elem[0].colorimetric = COLORIMETRIC_PRINT;
  }

  _image.elem[0].encoding = 0;
  _image.elem[0].data_offset = 2048;
  _image.elem[0].line_padding = 0;
  _image.elem[0].elem_padding = 0;
  
  // File image tags.

  const List<Str> & label = tag::label_tag();
  const List<Str> & label_dpx = io_dpx::label_tag();
  Str tmp;
  
  str::cstr(file, _file.name, 100, false);

  tmp = tag[label[tag::TIME]];
  if (tmp.size()) str::cstr(tmp, _file.time, 24, false);

  tmp = tag[label[tag::CREATOR]];
  if (tmp.size()) str::cstr(tmp, _file.creator, 100, false);

  tmp = tag[label[tag::PROJECT]];
  if (tmp.size()) str::cstr(tmp, _file.project, 200, false);

  tmp = tag[label[tag::COPYRIGHT]];
  if (tmp.size()) str::cstr(tmp, _file.copyright, 200, false);
  
  _file.encryption_key = 0;
  
  // Source image tags.
  
  tmp = tag[label_dpx[TAG_SOURCE_OFFSET]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _source.offset[0] = str::str_to_int(list[0]);
      _source.offset[1] = str::str_to_int(list[1]);
    }
  }

  tmp = tag[label_dpx[TAG_SOURCE_CENTER]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _source.center[0] = str::str_to_float(list[0]);
      _source.center[1] = str::str_to_float(list[1]);
    }
  }

  tmp = tag[label_dpx[TAG_SOURCE_SIZE]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _source.size[0] = str::str_to_int(list[0]);
      _source.size[1] = str::str_to_int(list[1]);
    }
  }
  
  tmp = tag[label_dpx[TAG_SOURCE_FILE]];
  if (tmp.size()) str::cstr(tmp, _source.file, 100, false);

  tmp = tag[label[TAG_SOURCE_TIME]];
  if (tmp.size()) str::cstr(tmp, _source.time, 24, false);

  tmp = tag[label_dpx[TAG_SOURCE_INPUT_DEVICE]];
  if (tmp.size()) str::cstr(tmp, _source.input_device, 32, false);

  tmp = tag[label_dpx[TAG_SOURCE_INPUT_SERIAL]];
  if (tmp.size()) str::cstr(tmp, _source.input_serial, 32, false);

  tmp = tag[label_dpx[TAG_SOURCE_BORDER]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (4 == list.size())
    {
      _source.border[0] = str::str_to_int(list[0]);
      _source.border[1] = str::str_to_int(list[1]);
      _source.border[2] = str::str_to_int(list[2]);
      _source.border[3] = str::str_to_int(list[3]);
    }
  }

  tmp = tag[label_dpx[TAG_SOURCE_PIXEL_ASPECT]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _source.pixel_aspect[0] = str::str_to_int(list[0]);
      _source.pixel_aspect[1] = str::str_to_int(list[1]);
    }
  }

  tmp = tag[label_dpx[TAG_SOURCE_SCAN_SIZE]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _source.scan_size[0] = str::str_to_float(list[0]);
      _source.scan_size[1] = str::str_to_float(list[1]);
    }
  }
  
  // Film image tags.

  tmp = tag[label[tag::KEYCODE]];
  if (tmp.size())
  {
    int id = 0, type = 0, prefix = 0, count = 0, offset = 0;
    time::str_to_keycode(tmp, &id, &type, &prefix, &count, &offset);
    str::cstr(str::int_to_str(id), _film.id, 2, false);
    str::cstr(str::int_to_str(type), _film.type, 2, false);
    str::cstr(str::int_to_str(offset), _film.offset, 2, false);
    str::cstr(str::int_to_str(prefix), _film.prefix, 6, false);
    str::cstr(str::int_to_str(count), _film.count, 4, false);
  }
  
  tmp = tag[label_dpx[TAG_FILM_FORMAT]];
  if (tmp.size()) str::cstr(tmp, _film.format, 32, false);

  tmp = tag[label_dpx[TAG_FILM_FRAME]];
  if (tmp.size()) _film.frame = str::str_to_int(tmp);

  tmp = tag[label_dpx[TAG_FILM_SEQUENCE]];
  if (tmp.size()) _film.sequence = str::str_to_int(tmp);

  tmp = tag[label_dpx[TAG_FILM_HOLD]];
  if (tmp.size()) _film.hold = str::str_to_int(tmp);

  tmp = tag[label_dpx[TAG_FILM_FRAME_RATE]];
  if (tmp.size()) _film.frame_rate = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_FILM_SHUTTER]];
  if (tmp.size()) _film.shutter = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_FILM_FRAME_ID]];
  if (tmp.size()) str::cstr(tmp, _film.frame_id, 32, false);

  tmp = tag[label_dpx[TAG_FILM_SLATE]];
  if (tmp.size()) str::cstr(tmp, _film.slate, 100, false);
  
  // TV image tags.

  tmp = tag[label[tag::TIMECODE]];
  if (tmp.size()) _tv.timecode = time::str_to_timecode(tmp);

  tmp = tag[label_dpx[TAG_TV_INTERLACE]];
  if (tmp.size()) _tv.interlace = str::str_to_int(tmp);

  tmp = tag[label_dpx[TAG_TV_FIELD]];
  if (tmp.size()) _tv.field = str::str_to_int(tmp);

  tmp = tag[label_dpx[TAG_TV_VIDEO_SIGNAL]];
  if (tmp.size()) _tv.video_signal = str::str_to_int(tmp);
  
  tmp = tag[label_dpx[TAG_TV_SAMPLE_RATE]];
  if (tmp.size())
  {
    const List<Str> list = str::split(tmp, ' ');
    if (2 == list.size())
    {
      _tv.sample_rate[0] = str::str_to_float(list[0]);
      _tv.sample_rate[1] = str::str_to_float(list[1]);
    }
  }
  
  tmp = tag[label_dpx[TAG_TV_FRAME_RATE]];
  if (tmp.size()) _tv.frame_rate = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_TIME_OFFSET]];
  if (tmp.size()) _tv.time_offset = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_GAMMA]];
  if (tmp.size()) _tv.gamma = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_BLACK_LEVEL]];
  if (tmp.size()) _tv.black_level = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_BLACK_GAIN]];
  if (tmp.size()) _tv.black_gain = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_BREAK_POINT]];
  if (tmp.size()) _tv.breakpoint = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_WHITE_LEVEL]];
  if (tmp.size()) _tv.white_level = str::str_to_float(tmp);

  tmp = tag[label_dpx[TAG_TV_INTEGRATION_TIMES]];
  if (tmp.size()) _tv.integration_times = str::str_to_float(tmp);
  
  // Write.
  
  dbg();
  
  mem::ENDIAN file_endian = mem::endian();
  if (MSB == endian) file_endian = mem::MSB;
  else if (LSB == endian) file_endian = mem::LSB;
  if (file_endian != mem::endian())
  {
    DJV_DBG_PRN("endian");
    io->endian(true);
    this->endian();
  }
  mem::copy(mem::MSB == file_endian ? magic[0] : magic[1], &_file.magic, 4);
  
  io->set(&_file, sizeof(File));
  io->set(&_image, sizeof(Image));
  io->set(&_source, sizeof(Source));
  io->set(&_film, sizeof(Film));
  io->set(&_tv, sizeof(Tv));
}

void Header::endian()
{
  mem::endian(&_file.image_offset, 1, 4);
  mem::endian(&_file.size, 1, 4);
  mem::endian(&_file.ditto_key, 1, 4);
  mem::endian(&_file.header_size, 1, 4);
  mem::endian(&_file.industry_header_size, 1, 4);
  mem::endian(&_file.user_header_size, 1, 4);
  mem::endian(&_file.encryption_key, 1, 4);
  
  mem::endian(&_image.orient, 1, 2);
  mem::endian(&_image.elem_size, 1, 2);
  mem::endian(_image.size, 2, 4);
  for (uint i = 0; i < 8; ++i)
  {
    mem::endian(&_image.elem[i].data_sign, 1, 4);
    mem::endian(&_image.elem[i].low_data, 1, 4);
    mem::endian(&_image.elem[i].low_quantity, 1, 4);
    mem::endian(&_image.elem[i].high_data, 1, 4);
    mem::endian(&_image.elem[i].high_quantity, 1, 4);
    mem::endian(&_image.elem[i].packing, 1, 2);
    mem::endian(&_image.elem[i].encoding, 1, 2);
    mem::endian(&_image.elem[i].data_offset, 1, 4);
    mem::endian(&_image.elem[i].line_padding, 1, 4);
    mem::endian(&_image.elem[i].elem_padding, 1, 4);
  }
  
  mem::endian(_source.offset, 2, 4);
  mem::endian(_source.center, 2, 4);
  mem::endian(_source.size, 2, 4);
  mem::endian(_source.border, 4, 2);
  mem::endian(_source.pixel_aspect, 2, 4);
  mem::endian(_source.scan_size, 2, 4);
  
  mem::endian(&_film.frame, 1, 4);
  mem::endian(&_film.sequence, 1, 4);
  mem::endian(&_film.hold, 1, 4);
  mem::endian(&_film.frame_rate, 1, 4);
  mem::endian(&_film.shutter, 1, 4);

  mem::endian(&_tv.timecode, 1, 4);
  mem::endian(&_tv.user_bits, 1, 4);
  mem::endian(_tv.sample_rate, 2, 4);
  mem::endian(&_tv.frame_rate, 1, 4);
  mem::endian(&_tv.time_offset, 1, 4);
  mem::endian(&_tv.gamma, 1, 4);
  mem::endian(&_tv.black_level, 1, 4);
  mem::endian(&_tv.black_gain, 1, 4);
  mem::endian(&_tv.breakpoint, 1, 4);
  mem::endian(&_tv.white_level, 1, 4);
  mem::endian(&_tv.integration_times, 1, 4);
}

void Header::dbg() const
{
  using info::dbg;
  
  DJV_DBG_OBJ("Header::dbg");
  
  DJV_DBG_PRN("file");
  DJV_DBG_PRN("  image offset = " << dbg(_file.image_offset));
  DJV_DBG_PRN("  version = " << dbg(_file.version, 8));
  DJV_DBG_PRN("  file size = " << dbg(_file.size));
  DJV_DBG_PRN("  ditto key = " << dbg(_file.ditto_key));
  DJV_DBG_PRN("  header size = " << dbg(_file.header_size));
  DJV_DBG_PRN("  industry header size = " << dbg(_file.industry_header_size));
  DJV_DBG_PRN("  user header size = " << dbg(_file.user_header_size));
  DJV_DBG_PRN("  name = " << dbg(_file.name, 100));
  DJV_DBG_PRN("  time = " << dbg(_file.time, 24));
  DJV_DBG_PRN("  creator = " << dbg(_file.creator, 100));
  DJV_DBG_PRN("  project = " << dbg(_file.project, 200));
  DJV_DBG_PRN("  copyright = " << dbg(_file.copyright, 200));
  DJV_DBG_PRN("  encryption key = " << dbg(_file.encryption_key));

  DJV_DBG_PRN("image");
  DJV_DBG_PRN("  orient = " << dbg(_image.orient));
  DJV_DBG_PRN("  element size = " << dbg(_image.elem_size));
  DJV_DBG_PRN("  size = " << dbg(_image.size[0]) << " " << dbg(_image.size[1]));
  for (int i = 0; i < _image.elem_size; ++i)
  {
    DJV_DBG_PRN("  element " << i);
    DJV_DBG_PRN("    data sign = " << dbg(_image.elem[i].data_sign));
    DJV_DBG_PRN("    low data = " << dbg(_image.elem[i].low_data));
    DJV_DBG_PRN("    low quantity = " << dbg(_image.elem[i].low_quantity));
    DJV_DBG_PRN("    high data = " << dbg(_image.elem[i].high_data));
    DJV_DBG_PRN("    high quantity = " << dbg(_image.elem[i].high_quantity));
    DJV_DBG_PRN("    descriptor = " << dbg(_image.elem[i].descriptor));
    DJV_DBG_PRN("    transfer = " << dbg(_image.elem[i].transfer));
    DJV_DBG_PRN("    colorimetric = " << dbg(_image.elem[i].colorimetric));
    DJV_DBG_PRN("    bit depth = " << dbg(_image.elem[i].bit_depth));
    DJV_DBG_PRN("    packing = " << dbg(_image.elem[i].packing));
    DJV_DBG_PRN("    encoding = " << dbg(_image.elem[i].encoding));
    DJV_DBG_PRN("    data offset = " << dbg(_image.elem[i].data_offset));
    DJV_DBG_PRN("    line padding = " << dbg(_image.elem[i].line_padding));
    DJV_DBG_PRN("    elem padding = " << dbg(_image.elem[i].elem_padding));
    DJV_DBG_PRN("    description = " << dbg(_image.elem[i].description, 32));
  }
  
  DJV_DBG_PRN("source");
  DJV_DBG_PRN("  offset = " << dbg(_source.offset[0]) << " " <<
    dbg(_source.offset[1]));
  DJV_DBG_PRN("  center = " << dbg(_source.center[0]) << " " <<
    dbg(_source.center[1]));
  DJV_DBG_PRN("  size = " << dbg(_source.size[0]) << " " <<
    dbg(_source.size[1]));
  DJV_DBG_PRN("  file = " << dbg(_source.file, 100));
  DJV_DBG_PRN("  time = " << dbg(_source.time, 24));
  DJV_DBG_PRN("  input device = " << dbg(_source.input_device, 32));
  DJV_DBG_PRN("  inputs serial = " << dbg(_source.input_serial, 32));
  DJV_DBG_PRN("  border = " << dbg(_source.border[0]) << " " <<
    dbg(_source.border[1]) << " " <<  dbg(_source.border[2]) << " " <<
    dbg(_source.border[3]));
  DJV_DBG_PRN("  pixel aspect = " << dbg(_source.pixel_aspect[0]) << " " <<
    dbg(_source.pixel_aspect[1]));
  DJV_DBG_PRN("  scan size = " << dbg(_source.scan_size[0]) << " " <<
    dbg(_source.scan_size[1]));

  DJV_DBG_PRN("film");
  DJV_DBG_PRN("  id = " << dbg(_film.id, 2));
  DJV_DBG_PRN("  type = " << dbg(_film.type, 2));
  DJV_DBG_PRN("  offset = " << dbg(_film.offset, 2));
  DJV_DBG_PRN("  prefix = " << dbg(_film.prefix, 6));
  DJV_DBG_PRN("  count = " << dbg(_film.count, 4));
  DJV_DBG_PRN("  format = " << dbg(_film.format, 32));
  DJV_DBG_PRN("  frame = " << dbg(_film.frame));
  DJV_DBG_PRN("  sequence = " << dbg(_film.sequence));
  DJV_DBG_PRN("  hold = " << dbg(_film.hold));
  DJV_DBG_PRN("  frame rate = " << dbg(_film.frame_rate));
  DJV_DBG_PRN("  shutter = " << dbg(_film.shutter));
  DJV_DBG_PRN("  frame id = " << dbg(_film.frame_id, 32));
  DJV_DBG_PRN("  slate = " << dbg(_film.slate, 100));

  DJV_DBG_PRN("tv");
  DJV_DBG_PRN("  timecode = " << dbg(_tv.timecode));
  DJV_DBG_PRN("  user bits = " << dbg(_tv.user_bits));
  DJV_DBG_PRN("  interlace = " << dbg(_tv.interlace));
  DJV_DBG_PRN("  field = " << dbg(_tv.field));
  DJV_DBG_PRN("  video signal = " << dbg(_tv.video_signal));
  DJV_DBG_PRN("  sample rate = " << dbg(_tv.sample_rate[0]) << " " <<
    dbg(_tv.sample_rate[1]));
  DJV_DBG_PRN("  frame rate = " << dbg(_tv.frame_rate));
  DJV_DBG_PRN("  time offset = " << dbg(_tv.time_offset));
  DJV_DBG_PRN("  gamma = " << dbg(_tv.gamma));
  DJV_DBG_PRN("  black level = " << dbg(_tv.black_level));
  DJV_DBG_PRN("  black gain = " << dbg(_tv.black_gain));
  DJV_DBG_PRN("  breakpoint = " << dbg(_tv.breakpoint));
  DJV_DBG_PRN("  white level = " << dbg(_tv.white_level));
  DJV_DBG_PRN("  integration times = " << dbg(_tv.integration_times));
}

}

void info_load(djv_base::Io * io, Info * info, Tag * tag, time::Speed * speed,
  bool * print) throw (Error)
{
  info::Header header;
  header.load(io, info, tag, speed, print);
}

void info_save(djv_base::Io * io, const Str & file, const Info & info,
  const Tag & tag, ENDIAN endian, io_cineon::COLOR color) throw (Error)
{
  info::Header header;
  header.save(io, file, info, tag, endian, color);
}

void info_update(djv_base::Io * io) throw (Error)
{
  const size_t size = io->position();
  
  io->position(12);
  io->set_u32(size);
}

}
}
