/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_info.h
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _DJV_IMAGE_IO_DPX_INFO_H
#define _DJV_IMAGE_IO_DPX_INFO_H

#include <djv_image/io_dpx.h>

namespace djv_image {
namespace io_dpx {
namespace info {

//------------------------------------------------------------------------------
// Constants

const char magic [][5] =
{
  "SDPX",
  "XPDS"
};

const float speed_min = 0.000001; // XXX

//------------------------------------------------------------------------------
// Options

enum ORIENT
{
  ORIENT_LEFT_RIGHT_TOP_BOTTOM,
  ORIENT_RIGHT_LEFT_TOP_BOTTOM,
  ORIENT_LEFT_RIGHT_BOTTOM_TOP,
  ORIENT_RIGHT_LEFT_BOTTOM_TOP,
  ORIENT_TOP_BOTTOM_LEFT_RIGHT,
  ORIENT_TOP_BOTTOM_RIGHT_LEFT,
  ORIENT_BOTTOM_TOP_LEFT_RIGHT,
  ORIENT_BOTTOM_TOP_RIGHT_LEFT
};

enum DESCRIPTOR
{
  DESCRIPTOR_USER = 0,
  DESCRIPTOR_R = 1,
  DESCRIPTOR_G = 2,
  DESCRIPTOR_B = 3,
  DESCRIPTOR_A = 4,
  DESCRIPTOR_L = 6,
  DESCRIPTOR_CHROMA = 7,
  DESCRIPTOR_Z = 8,
  DESCRIPTOR_COMPOSITE_VIDEO = 9,
  DESCRIPTOR_RGB = 50,
  DESCRIPTOR_RGBA = 51,
  DESCRIPTOR_ABGR = 52,
  DESCRIPTOR_CBYCRY = 100,
  DESCRIPTOR_CBYACRYA = 101,
  DESCRIPTOR_CBYCR = 102,
  DESCRIPTOR_CBYCRA = 103,
  DESCRIPTOR_USER_2 = 150,
  DESCRIPTOR_USER_3 = 151,
  DESCRIPTOR_USER_4 = 152,
  DESCRIPTOR_USER_5 = 153,
  DESCRIPTOR_USER_6 = 154,
  DESCRIPTOR_USER_7 = 155,
  DESCRIPTOR_USER_8 = 156
};

enum TRANSFER
{
  TRANSFER_USER,
  TRANSFER_PRINT,
  TRANSFER_LINEAR,
  TRANSFER_LOG,
  TRANSFER_VIDEO,
  TRANSFER_SMPTE_274M,
  TRANSFER_ITU_R_709_4,
  TRANSFER_ITU_R_601_5_B_OR_G,
  TRANSFER_ITU_R_601_5_M,
  TRANSFER_NTSC,
  TRANSFER_PAL,
  TRANSFER_Z,
  TRANSFER_Z_HOMOGENEOUS
};

enum COLORIMETRIC
{
  COLORIMETRIC_USER = 0,
  COLORIMETRIC_PRINT = 1,
  COLORIMETRIC_VIDEO = 4,
  COLORIMETRIC_SMPTE_274M = 5,
  COLORIMETRIC_ITU_R_709_4 = 6,
  COLORIMETRIC_ITU_R_601_5_B_OR_G = 7,
  COLORIMETRIC_ITU_R_601_5_M = 8,
  COLORIMETRIC_NTSC = 9,
  COLORIMETRIC_PAL = 10
};

enum COMPONENTS
{
  PACK,
  TYPE_A,
  TYPE_B
};

//------------------------------------------------------------------------------
// Header

struct Header
{
  Header();
  
  void load(
    djv_base::io::Io *,
    Info *,
    Tag *,
    time::Speed *,
    bool * print
  ) throw (Error);
  
  void save(
    djv_base::io::Io *,
    const Str & file,
    const Info &,
    const Tag &,
    ENDIAN,
    io_cineon::COLOR
  ) throw (Error);
  
  void endian();
  
  void dbg() const;

private:
  
  struct File
  {
    uint32_t magic;
    uint32_t image_offset;
    char version [8];
    uint32_t size;
    uint32_t ditto_key;
    uint32_t header_size;
    uint32_t industry_header_size;
    uint32_t user_header_size;
    char name [100];
    char time [24];
    char creator [100];
    char project [200];
    char copyright [200];
    uint32_t encryption_key;
    uint8_t _pad [104];
  } _file;
  
  struct Image
  {
    uint16_t orient;
    uint16_t elem_size;
    uint32_t size [2];
    
    struct Elem
    {
      uint32_t data_sign;
      uint32_t low_data;
      float low_quantity;
      uint32_t high_data;
      float high_quantity;
      uint8_t descriptor;
      uint8_t transfer;
      uint8_t colorimetric;
      uint8_t bit_depth;
      uint16_t packing, encoding;
      uint32_t data_offset;
      uint32_t line_padding;
      uint32_t elem_padding;
      char description [32];
    } elem [8];

    uint8_t _pad [52];
  } _image;

  struct Source
  {
    uint32_t offset [2];
    float center [2];
    uint32_t size [2];
    char file [100];
    char time [24];
    char input_device [32];
    char input_serial [32];
    uint16_t border [4];
    uint32_t pixel_aspect [2];
    float scan_size [2];
    uint8_t _pad [20];
  } _source;

  struct Film
  {
    char id [2];
    char type [2];
    char offset [2];
    char prefix [6];
    char count [4];
    char format [32];
    uint32_t frame;
    uint32_t sequence;
    uint32_t hold;
    float frame_rate;
    float shutter;
    char frame_id [32];
    char slate [100];
    uint8_t _pad [56];
  } _film;

  struct Tv
  {
    uint32_t timecode;
    uint32_t user_bits;
    uint8_t interlace;
    uint8_t field, video_signal;
    uint8_t _pad;
    float sample_rate [2];
    float frame_rate;
    float time_offset;
    float gamma;
    float black_level;
    float black_gain;
    float breakpoint;
    float white_level;
    float integration_times;
    uint8_t _pad2 [76];
  } _tv;
};

//------------------------------------------------------------------------------
// Functions

void zero(char *, int size);

bool valid(const uint8_t *);
bool valid(const uint16_t *);
bool valid(const uint32_t *);
bool valid(const float *);
bool valid(const char *, int size);

Str to_str(const char *, int size);

//------------------------------------------------------------------------------
// Debugging.

Str dbg(uint8_t in);
Str dbg(uint16_t in);
Str dbg(uint32_t in);
Str dbg(float in);
Str dbg(const char * in, int size);

}
}
}

#endif
