/*------------------------------------------------------------------------------
 lib/djv_image/io_sgi_data.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_sgi.h>

namespace djv_image {
namespace io_sgi {

namespace {

bool load_u8(
  const uint8_t * in,
  const uint8_t * end,
  uint8_t * out,
  int size
) {
  DJV_DBG_OBJ("load_u8");
  DJV_DBG_PRN("size = " << size);
  
  const uint8_t * const out_end = out + size;
  while (out < out_end)
  {
    // Information.
    
    if (in > end) return false;

    const int count = *in & 0x7f;
    const bool run = ! (*in & 0x80);
    const int length = run ? 1 : count;
    DJV_DBG_PRN("count = " << count);
    DJV_DBG_PRN("  run = " << run);
    DJV_DBG_PRN("  length = " << length);
    
    ++in;

    // Unpack.
    
    if (in + length > end) return false;
    
    if (run)
    {
      for (int j = 0; j < count; ++j, ++out)
        *out = *in;
      ++in;
    }
    else
    {
      for (int j = 0; j < length; ++j, ++in, ++out)
        *out = *in;
    }
  }
  
  return true;
}

bool load_u16(
  const uint16_t * in,
  const uint16_t * end,
  uint16_t * out,
  int size
) {
  const uint16_t * const out_end = out + size;
  while (out < out_end)
  {
    if (in > end) return false;
    const int count = *in & 0x7f;
    const bool run = ! (*in & 0x80);
    const int length = run ? 1 : count;
    ++in;

    if (in + length > end) return false;
    if (run)
    {
      for (int j = 0; j < count; ++j, ++out)
        *out = *in;
      ++in;
    }
    else
    {
      for (int j = 0; j < length; ++j, ++in, ++out)
        *out = *in;
    }
  }
  
  return true;
}

template<typename T>
size_t save(const void * in, void * out, int size)
{
  DJV_DBG_OBJ("save");
  DJV_DBG_PRN("size = " << size);
  
  const T * in_p = reinterpret_cast<const T *>(in);
  T * out_p = reinterpret_cast<T *>(out);
  const T * const end = in_p + size;
  
  while (in_p < end)
  {
    // Pixel runs.
    
    const int min = 3;
    const int max = math::min(0x7f, int(end - in_p));
    int count = 1, match = 1;
    for (; count < max; ++count)
    {
      if (in_p[count] == in_p[count - 1])
      {
        ++match;
        if (min == match && count >= min)
        {
          count -= min - 1;
          match = 1;
          break;
        }
      }
      else
      {
        if (match >= min) break;
        match = 1;
      }
    }
    const bool run = match > min;
    const int length = run ? 1 : count;
    DJV_DBG_PRN("count = " << count);
    DJV_DBG_PRN("  run = " << run);
    DJV_DBG_PRN("  length = " << length);
    
    // Information.
    
    out_p[0] = (count & 0x7f) | ((! run) << 7);
    
    // Pack.
    
    for (int i = 0; i < length; ++i)
      out_p[1 + i] = in_p[i];
    
    out_p += 1 + length;
    in_p += count;
  }
  
  // Cap the end.
  
  *out_p++ = 0;
  
  const size_t r = (out_p - reinterpret_cast<T *>(out)) * sizeof(T);
  DJV_DBG_PRN("r = " << int(r));
  return r;
}

}

bool rle_load(
  const uint8_t * in,
  const uint8_t * end,
  void * out, int size,
  int bytes
) {
  switch (bytes)
  {
    case 1: return load_u8(
      in, end, reinterpret_cast<uint8_t *>(out), size);

    case 2: return load_u16(
      reinterpret_cast<const uint16_t *>(in),
      reinterpret_cast<const uint16_t *>(end),
      reinterpret_cast<uint16_t *>(out), size);
  }
  return false;
}

size_t rle_save(const void * in, void * out, int size, int bytes)
{
  switch (bytes)
  {
    case 1: return save<uint8_t>(in, out, size) * bytes;
    case 2: return save<uint16_t>(in, out, size) * bytes;
  }
  return 0;
}

}
}
