/*------------------------------------------------------------------------------
 lib/djv_image/io_sgi_load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_sgi.h>

namespace djv_image {
namespace io_sgi {

Load::Load()
{
  _io.endian(mem::endian() != mem::MSB);
}

io::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return io_sgi::name; }

const List<Str> & Load::extension_list() const
  { return io_sgi::extension_list; }

void Load::open(const File & in, Info * info, Tag *, Time * time) throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start()), info);
  
  if (file::SEQ == _file.type)
    time->list = _file.seq.list;
}

void Load::open(const File & in, List<Info> * info, Tag * tag, Time * time)
  throw (Error)
{
  io::Load::open(in, info, tag, time);
}

void Load::_open(const File & in, Info * info) throw (Error)
{
  DJV_DBG_OBJ("Load::_open");
  DJV_DBG_PRN("in = " << in);
  
  _io.open(in, djv_base::io::READ);
  
  info->name = in;
  info_load(&_io, info, &_compress);

  // Scanline tables.
  
  if (_compress)
  {
    const int table_size = info->size.y * pixel::channels(info->pixel);
    DJV_DBG_PRN("rle table size = " << table_size);
  
    _rle_offset.size(table_size);
    _rle_size.size(table_size);

    _io.get_u32(_rle_offset(), table_size);
    _io.get_u32(_rle_size(), table_size);
  }
}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  // Open.
  
  Info info;
  _open(_file.get(frame != -1 ? frame : _file.seq.start()), &info);
  
  // Read.
  
  _io.readahead();

  const off_t position = _io.position();
  const size_t size = _io.size() - position;
  
  const int channels = pixel::channels(info.pixel);
  const int bytes = pixel::channel_bytes(info.pixel);

  if (! _compress)
  {
    if (1 == bytes)
    {
      const uint8_t * p = _io.mmap_p();
      _io.seek(data::bytes_data(info));
      _tmp.set(info, p);
    }
    else
    {
      _tmp.set(info);
      _io.get(_tmp.data(), size / bytes, bytes);
    }
  }
  else
  {
    _tmp.set(info);
    
    mem::Buffer<uint8_t> tmp(size);
    _io.get(tmp(), size / bytes, bytes);

    const uint8_t * in_p = tmp();
    const uint8_t * end = in_p + size;
    uint8_t * out_p = _tmp.data();

    for (int c = 0; c < channels; ++c)
    {
      DJV_DBG_PRN("channel = " << c);

      for (int y = 0; y < info.size.y; ++y, out_p += info.size.x * bytes)
      {
        DJV_DBG_PRN("y = " << y);
        
        if (
          ! rle_load(
            in_p + _rle_offset()[y + info.size.y * c] - position, end, out_p,
            info.size.x, bytes)
        )
          throw Error(name(), io::error_read);
      }
    }
  }
  
  // Interleave channels.
  
  info.size = data::proxy_scale(info.size, proxy);
  info.proxy = proxy;
  _image.set(info);
  data::planar_interleave(_tmp, &_image, proxy);
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}
