/*------------------------------------------------------------------------------
 lib/djv_image/io_sgi_save.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_sgi.h>

namespace djv_image {
namespace io_sgi {

Save::Save()
{
  _io.endian(mem::endian() != mem::MSB);
}

io::Plugin * Save::copy() const
{
  Save * out = new Save;
  out->_option = _option;
  return out;
}

const Str & Save::name() const { return io_sgi::name; }

const List<Str> & Save::extension_list() const
  { return io_sgi::extension_list; }

bool Save::option(const Str & in, Str * data)
{
try
{
  if (str::compare(in, option_list()[SAVE_COMPRESS])) *data >> _option.compress;
}
catch (Str) { return false; }
  return true;
}

Str Save::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[SAVE_COMPRESS])) out << _option.compress;
  return out;
}

const List<Str> & Save::option_list() const
  { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &,
  const Time & time) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  DJV_DBG_PRN("in = " << in);
  
  _file = in;
  if (time.list.size() > 1) _file.type = file::SEQ;
  
  _info = Info();
  _info.size = info.size;
  pixel::TYPE type = pixel::type(info.pixel);
  switch (type)
  {
    case pixel::U10:
    case pixel::F16: 
    case pixel::F32: type = pixel::U16; break;
    default: break;
  }
  _info.pixel = pixel::pixel(pixel::format(info.pixel), type);
  _info.endian = mem::MSB;
  DJV_DBG_PRN("info = " << _info);

  _image.set(_info);
}

void Save::_open(const File & in) throw (Error)
{
  DJV_DBG_OBJ("Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv_base::io::WRITE);

  info_save(&_io, _info, _option.compress);
  
  // Scanline tables.

  if (_option.compress)
  {
    const int table_size = _info.size.y * pixel::channels(_info.pixel);
    DJV_DBG_PRN("rle table size = " << table_size);
    
    _rle_offset.size(table_size);
    _rle_size.size(table_size);
    
    _io.seek(table_size * 2 * 4);
  }
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("compress = " << _option.compress);

  // Open.
  
  _open(_file.get(frame));

  // Write.
  
  const Data * p = &in;
  if (in.info() != _info)
  {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl_data::copy(in, &_image);
    p = &_image;
  }
  
  _tmp.set(p->info());

  const int w = _tmp.w(), h = _tmp.h();
  const int channels = pixel::channels(_tmp.pixel());
  const int bytes = pixel::channel_bytes(_tmp.pixel());

  // Deinterleave channels.
  
  data::planar_deinterleave(*p, &_tmp);
  
  // Write data.

  if (! _option.compress)
  {
    _io.set(_tmp.data(), _tmp.bytes_data() / bytes, bytes);
  }
  else
  {
    mem::Buffer<uint8_t> scanline(w * bytes * 2);

    for (int c = 0; c < channels; ++c)
      for (int y = 0; y < h; ++y)
      {
        const size_t size = rle_save(_tmp.data() + (c * h + y) * w * bytes,
          scanline(), w, bytes);

        _rle_offset()[y + c * h] = uint32_t(_io.position());
        _rle_size()[y + c * h] = uint32_t(size);

        _io.set(scanline.data(), size / bytes, bytes);
      }

    _io.position(512);
    _io.set_u32(_rle_offset(), h * channels);
    _io.set_u32(_rle_size(), h * channels);
  }

  _io.close();
}

}
}
