/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_dialog.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "file_browser.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs.h>
#include <djv_ui/shortcut.h>
#include <djv_ui/style.h>
#include <djv_base/dir.h>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace file_browser {

Dialog::Dialog() :
  dialog::Base(dialog_title),
  sig(this),
  _image(prefs()->image()),
  _seq(prefs()->seq()),
  _hidden(prefs()->hidden()),
  _sort(prefs()->sort()),
  _sort_reverse(prefs()->sort_reverse()),
  _sort_dir(prefs()->sort_dir()),
  _pin(prefs()->pin()),
  _p(new _Dialog)
{
  DJV_DBG_OBJ("Dialog::Dialog");
  
  // Widgets.

  _p->menu = new menu::Widget;
  _p->widget = new text::Edit;
  _p->dir = new text::Edit;

  _p->up = new button::Tool("dir_up");
  _p->prev = new button::Tool("dir_prev");
  _p->reload = new button::Tool("dir_reload");

  _p->browser = new Browser;
  _p->columns = new columns::Widget;
  _p->columns->name_list(label_sort());
  
  _p->pin = new button::Check(dialog::label_pin);
  button::Push * ok_button = new button::Push(dialog::label_ok);
  button::Push * close_button = new button::Push(dialog::label_close);
  
  group::Frame * frame = new group::Frame;
  frame->edges(false, false, true, false);
  group::Frame * browser_frame = new group::Frame;
  browser_frame->frame(FL_THIN_DOWN_FRAME);
  
  // Layout.

  Layout_V * layout_window = new Layout_V(this);
  layout_window->spacing(0);
  layout_window->margin(0);
  layout_window->add(_p->menu);
  layout_window->add(frame);
  layout_window->stretch(frame);
  Layout_V * layout = new Layout_V(frame);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(_p->dir);
  layout_h->stretch(_p->dir);
  Layout_H * layout_tool = new Layout_H(layout_h);
  layout_tool->spacing(0);
  layout_tool->margin(0);
  layout_tool->add(_p->up);
  layout_tool->add(_p->prev);
  layout_tool->add(_p->reload);
  
  layout->add(browser_frame);
  layout->stretch(browser_frame);
  Layout_V * layout_v = new Layout_V(browser_frame);
  layout_v->spacing(0);
  layout_v->margin(0);
  layout_v->add(_p->columns);
  layout_v->add(_p->browser);
  layout_v->stretch(_p->browser);
  
  layout->add(_p->widget);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(_p->pin);
  layout_h->add_spacer(-1, true);
  layout_h->add(ok_button);
  layout_h->add(close_button);
  
  // Initialize.
  
  _dir = djv_base::file::path_fix(".");
  _dir_prev = _dir;
  
  //dir_update();
  widget_update();
  menu_update();
  _p->pin->set(_pin);
  
  size(Vec2i(600, 500));

  fl_font(style::global()->font(), style::global()->font_size());
  _p->columns->size_list(columns_size_list());
  _p->browser->column_list(_p->columns->size_list());
  
  // Callbacks.

  _p->widget->sig.set(this, widget_callback);
  _p->widget->action_sig.set(this, widget_action_callback);
  _p->dir->sig.set(this, dir_callback);
  _p->columns->sig.set(this, columns_callback);
  _p->columns->action_sig.set(this, columns_action_callback);
  _p->browser->sig.set(this, browser_callback);
  _p->browser->action_sig.set(this, browser_action_callback);

  _p->up->sig.set(this, up_callback);
  _p->prev->sig.set(this, prev_callback);
  _p->reload->sig.set(this, reload_callback);

  _p->pin->sig.set(this, pin_callback);
  ok_button->sig.set(this, ok_callback);
  close_button->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::set(const Str & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ("Dialog::set");
  DJV_DBG_PRN("in = " << in);

  _value = in;

  widget_update();
}

File Dialog::get() const
{
  DJV_DBG_OBJ("Dialog::get");
  DJV_DBG_PRN("value = " << _value);
  DJV_DBG_PRN("dir = " << _dir);
  
  File out = _value;
  
  Str path = out.path;
  if (path.size())
  {
    if (! djv_base::file::path_is_absolute(path))
      path = _dir + path;

    out.path = djv_base::file::path_fix(path);
  }
  else
  {
    out.path = _dir;
  }
  
  out.stat();
  if (_seq && out.seq_valid()) out.type = djv_base::file::SEQ;
  DJV_DBG_PRN("out = " << out);
  return out;
}

void Dialog::dir(const Str & in)
{
  const Str tmp = djv_base::file::path_fix(in);
  //if (tmp == _dir) return;

  DJV_DBG_OBJ("Dialog::dir");
  DJV_DBG_PRN("in = " << tmp);
  
  _value = Str();
  _dir_prev = _dir;
  _dir = tmp;
  
  dir_update();
  widget_update();
}

const Str & Dialog::dir() const { return _dir; }

void Dialog::image(IMAGE in)
{
  if (in == _image) return;
  
  _image = in;
  prefs()->image(_image);
  
  dir_update();
  menu_update();
}

IMAGE Dialog::image() const { return _image; }

void Dialog::seq(file::SEQ_COMPRESS in)
{
  if (in == _seq) return;
  
  _seq = in;
  prefs()->seq(_seq);
  
  dir_update();
  menu_update();
}

djv_base::file::SEQ_COMPRESS Dialog::seq() const { return _seq; }

void Dialog::hidden(bool in)
{
  if (in == _hidden) return;
  
  _hidden = in;
  prefs()->hidden(_hidden);
  
  dir_update();
  menu_update();
}

bool Dialog::hidden() const { return _hidden; }

void Dialog::sort(SORT in)
{
  if (in == _sort) return;
  
  _sort = in;
  prefs()->sort(_sort);
  
  dir_update();
  menu_update();
}

SORT Dialog::sort() const { return _sort; }

void Dialog::sort_reverse(bool in)
{
  if (in == _sort_reverse) return;
  
  _sort_reverse = in;
  prefs()->sort_reverse(_sort_reverse);
  
  dir_update();
  menu_update();
}

bool Dialog::sort_reverse() const { return _sort_reverse; }

void Dialog::sort_dir(bool in)
{
  if (in == _sort_dir) return;
  
  _sort_dir = in;
  prefs()->sort_dir(_sort_dir);
  
  dir_update();
  menu_update();
}

bool Dialog::sort_dir() const { return _sort_dir; }

void Dialog::pin(bool in)
{
  if (in == _pin) return;
  
  _pin = in;
  prefs()->pin(_pin);
  
  _p->pin->set(_pin);
}

bool Dialog::pin() const { return _pin; }

void Dialog::show()
{
  DJV_DBG_OBJ("Dialog::show");
  
  if (! shown())
  {
    _value = Str();
  
    dir_update();
    widget_update();

    _p->widget->take_focus();
  }

  dialog::Base::show();
}

void Dialog::dirty()
{
  using shortcut::serialize;
  
  DJV_DBG_OBJ("Dialog::dirty");
  
  dialog::Base::dirty();
  
  const List<shortcut::Shortcut> & shortcut = prefs()->shortcut();

  _p->widget->tooltip(tooltip_widget);
  _p->dir->tooltip(tooltip_dir);
  
  _p->up->tooltip(str::Format(tooltip_up).
    arg(serialize(shortcut[Prefs::UP].value)));
  _p->prev->tooltip(str::Format(tooltip_prev).
    arg(serialize(shortcut[Prefs::PREV].value)));
  _p->reload->tooltip(str::Format(tooltip_reload).
    arg(serialize(shortcut[Prefs::RELOAD].value)));

  if (visible() && shown()) dir_update();
  menu_update();
}

void Dialog::dir_update()
{
  DJV_DBG_OBJ("Dialog::dir_update");
  DJV_DBG_PRN("dir = " << _dir);
  
  callbacks(false);
  
  _p->dir->set(_dir);
  
  base::Cursor cursor(FL_CURSOR_WAIT);
  
  const Dir dir(_dir);
  List<File> list = dir.list();
  djv_base::file::seq_compress(&list, _seq);
  if (! _hidden) djv_base::file::filter(&list, djv_base::file::FILTER_HIDDEN);
  djv_base::file::SORT sort = djv_base::file::SORT(0);
  switch (_sort)
  {
    case SORT_NAME: sort = djv_base::file::SORT_NAME; break;
    case SORT_SIZE: sort = djv_base::file::SORT_SIZE; break;
#if ! defined(WIN32)
    case SORT_USER: sort = djv_base::file::SORT_USER; break;
#endif
    case SORT_TIME: sort = djv_base::file::SORT_TIME; break;
    default: break;
  }
  djv_base::file::sort(&list, sort, _sort_reverse);
  if (_sort_dir) list = djv_base::file::sort_dir(list);
  if (dir.valid()) list.push_front(File(_dir + "/" + ".."));
  
  _p->columns->sort(_sort);
  _p->columns->sort_reverse(_sort_reverse);
  _p->browser->set(list, _image);

  callbacks(true);
}

void Dialog::widget_update()
{
  DJV_DBG_OBJ("Dialog::widget_update");
  DJV_DBG_PRN("value = " << _value);
  
  callbacks(false);
  
  _p->widget->set(_value);
    
  callbacks(true);
}

}
}

