/*------------------------------------------------------------------------------
 lib/djv_ui/label.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/label.h>
#include <djv_ui/style.h>
#include <FL/Fl_Image.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace label {

Widget::Widget(const Str & in) :
  _value_width(0)
{
  box(FL_FLAT_BOX);
  //text_font(style::global()->font_bold());

  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  set(in);
}

Widget::~Widget()
{}

void Widget::set(const Str & in)
{
  if (in == _value) return;

  DJV_DBG_OBJ("Widget::set");
  DJV_DBG_PRN("in = " << in);

  _value = in;
  
  const Vec2i size = size_hint();
  dirty();
  if (size != size_hint()) update();
  redraw();
}

const Str & Widget::get() const { return _value; }

namespace {

inline int offset(layout::ALIGN in, int size, int size_hint)
{
  int out = 0;
  switch (in)
  {
    case layout::LEFT: break;
    case layout::CENTER: out += size / 2 - size_hint / 2; break;
    case layout::RIGHT: out += size - size_hint; break;
  }
  return out;
}

}

void Widget::draw()
{
  widget::Widget::draw();

  const Box2i & geom = this->geom();
  
  fl_font(text_font(), style::global()->font_size());
  fl_color(text_color());
  fl_draw(
    _value.c_str(),
    geom.x + offset(align()[0], geom.w, _value_width),
    geom.y + offset(align()[1], geom.h, fl_height()) + fl_height() -
      fl_descent());
}

void Widget::size_str(const Str & in)
{
  _size_str = in;
}

const Str & Widget::size_str() const { return _size_str; }

void Widget::dirty()
{
  widget::Widget::dirty();

  fl_font(text_font(), style::global()->font_size());
  _value_width = base::str_width(_value);
  size_hint(Vec2i(
    math::max(_value_width, base::str_width(_size_str)),
    fl_height()));
}

layout::Item * Widget::group_align() { return 0; }
layout::Item * Widget::label_align() { return this; }

Text::Text(const Str & in) :
  _word_size_max(0),
  _space_size(0),
  _text_height(0)
{
  align(layout::LEFT, layout::CENTER);
  
  set(in);
}

Text::~Text()
{}

void Text::set(const Str & in)
{
  if (in == _value) return;

  DJV_DBG_OBJ("Text::set");
  DJV_DBG_PRN("in = " << in);

  _value = in;
  
  _word_list.clear();
  List<Str> tmp;
  if (_value.size())
    tmp = str::split(_value, '\n', true);
  for (uint i = 0; i < tmp.size(); ++i)
  {
    if (tmp[i].size())
      _word_list += str::split(tmp[i], ' ', true);
    if (i < tmp.size() - 1)
      _word_list += "\n";
  }
  _word_size_list.resize(_word_list.size() + 1);
  
  update();
  redraw();
}

const Str & Text::get() const { return _value; }

void Text::draw()
{
  widget::Widget::draw();

  const Box2i & geom = this->geom();

  fl_font(text_font(), style::global()->font_size());
  fl_color(FL_FOREGROUND_COLOR);
  
  int x = 0, y = 0;
  for (uint i = 0; i < _word_list.size(); ++i)
  {
    if (_word_list[i] != "\n")
      fl_draw(
        _word_list[i].c_str(),
        geom.x + offset(align()[0], geom.w, _text_hint.x) + x,
        geom.y + offset(align()[1], geom.h, _text_hint.y) + y +
          fl_height() - fl_descent()
      );
    x += _word_size_list[i];
    if ("\n" == _word_list[i] || ((x + _word_size_list[i + 1]) > (geom.w - 1)))
    {
      x = 0;
      y += _text_height;
    }
  }
}

void Text::dirty()
{
  DJV_DBG_OBJ("Text::dirty");
  
  widget::Widget::dirty();

  fl_font(text_font(), style::global()->font_size());
  
  _word_size_max = 0;
  uint i = 0;
  for (; i < _word_list.size(); ++i)
  {
    _word_size_list[i] = _word_list[i] != "\n" ?
      base::str_width(_word_list[i] + " ") : 0;
    _word_size_max = math::max(_word_size_max, _word_size_list[i]);
  }
  _word_size_list[i] = 0;
  _space_size = base::str_width(" ");
  _text_height = fl_height();
  
  _text_hint = text_hint(geom().size);
  _size_min = Vec2i(
    math::max(_word_size_max, style::global()->size_text()),
    _text_hint.y
  );
  DJV_DBG_PRN("text = " << _text_hint);
  DJV_DBG_PRN("min = " << _size_min);
  
  if (_text_hint != size_hint())
  {
    DJV_DBG_PRN("update");
    size_hint(_text_hint);
    update();
  }
}

void Text::geom(const Box2i & in)
{
  DJV_DBG_OBJ("Text::geom");
  DJV_DBG_PRN("in = " << in);
  
  const Box2i geom = this->geom();
  widget::Widget::geom(in);
  if (geom != this->geom())
  {
    const Vec2i size = text_hint(in.size);
    const Vec2i hint = size_hint();
    DJV_DBG_PRN("size = " << size);
    DJV_DBG_PRN("hint = " << hint);
    if (size != hint) dirty();
  }
}

const Box2i & Text::geom() const { return widget::Widget::geom(); }
const Vec2i & Text::size_min() const { return _size_min; }
layout::Item * Text::group_align() { return 0; }
layout::Item * Text::label_align() { return this; }

Vec2i Text::text_hint(const Vec2i & in) const
{
  Vec2i out;
  int _x = 0, _y = _text_height;
  for (uint i = 0; i < _word_list.size(); ++i)
  {
    _x += _word_size_list[i];
    if ("\n" == _word_list[i] || ((_x + _word_size_list[i + 1]) > (in.x - 1)))
    {
      out.x = math::max(out.x, _x);
      _x = 0;
      _y += _text_height;
    }
  }
  out.y = _y;
  return out;
}

Image::Image(Fl_Image * in) :
  _value(in)
{}

Image::~Image()
{}

void Image::set(Fl_Image * in)
{
  _value = in;

  update();
  redraw();
}

Fl_Image * Image::get() const { return _value; }

void Image::draw()
{
  widget::Widget::draw();
  
  const Box2i & geom = this->geom();
  
  if (_value)
    _value->draw(
      geom.x + (geom.w - _value->w()) / 2,
      geom.y + (geom.h - _value->h()) / 2);
}

void Image::dirty()
{
  widget::Widget::dirty();
  
  size_hint(_value ? Vec2i(_value->w(), _value->h()) : Vec2i());
}

layout::Item * Image::group_align() { return 0; }
layout::Item * Image::label_align() { return this; }

}
}

