/*------------------------------------------------------------------------------
 plugin/djv_png/load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"

namespace djv_png {
namespace io {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

Load::Load() :
  _f(0),
  _png(0),
  _png_info(0),
  _png_info_end(0)
{}

Load::~Load()
{
  _close();
}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_png::name; }

const List<Str> & Load::extension_list() const
  { return djv_png::extension_list; }

void Load::open(const File & in, Info * info, Tag *, djv_image::Time * time)
  throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start()), info);

  if (file::SEQ == _file.type)
    time->list = _file.seq.list;
}

void Load::open(const File & in, List<Info> * info, Tag * tag,
  djv_image::Time * time) throw (Error)
{
  djv_image::io::Load::open(in, info, tag, time);
}

namespace {

bool open(FILE * f, png_structp png, png_infop * png_info,
  png_infop * png_info_end)
{
  if (setjmp(png_jmpbuf(png))) return false;
  
  *png_info = png_create_info_struct(png);
  if (! *png_info) return false;
  *png_info_end = png_create_info_struct(png);
  if (! *png_info_end) return false;
  
  uint8_t tmp [8];
  if (::fread(tmp, 8, 1, f) != 1) return false;
  if (png_sig_cmp(tmp, 0, 8)) return false;
  png_init_io(png, f);
  png_set_sig_bytes(png, 8);
  png_read_info(png, *png_info);
  
  if (png_get_interlace_type(png, *png_info) != PNG_INTERLACE_NONE)
    return false;
  png_set_expand(png);
  png_set_gray_1_2_4_to_8(png);
  png_set_palette_to_rgb(png);
  png_set_tRNS_to_alpha(png);
    
  return true;
}

}

void Load::_open(const File & in, Info * info) throw (Error)
{
  DJV_DBG_OBJ("Load::_open");
  DJV_DBG_PRN("in = " << in);
  
  _close();

  // Initialize libpng.
  
  _png = png_create_read_struct(
    PNG_LIBPNG_VER_STRING, 0, djv_png_error, djv_png_warning);
  if (! _png)
    throw Error(name(), _png_error.msg);

  _png->error_ptr = &_png_error;

  SNPRINTF(_png_error.msg, str::cstr_size, djv_image::io::error_open.c_str());
  
  // Open.
  
  _f = ::fopen(in.get().c_str(), "rb");
  if (! _f)
    throw Error(name(), djv_image::io::error_open);

  if (! io::open(_f, _png, &_png_info, &_png_info_end))
    throw Error(name(), _png_error.msg);

  // Information.
  
  info->name = in;
  
  info->size = Vec2i(
    png_get_image_width(_png, _png_info),
    png_get_image_height(_png, _png_info));
  
  int channels = png_get_channels(_png, _png_info);
  if (png_get_color_type(_png, _png_info) == PNG_COLOR_TYPE_PALETTE)
    channels = 3;
  if (png_get_valid(_png, _png_info, PNG_INFO_tRNS))
    ++channels;
  DJV_DBG_PRN("channels = " << channels);
  
  int bit_depth = png_get_bit_depth(_png, _png_info);
  if (bit_depth < 8) bit_depth = 8;
  DJV_DBG_PRN("bit depth = " << bit_depth);
  
  if (bit_depth >= 16 && mem::LSB == mem::endian())
    png_set_swap(_png);
  
  if (! pixel::pixel_(channels, bit_depth, false, &info->pixel))
    throw Error(name(), djv_image::io::error_unsupported);
}

void Load::_close()
{
  if (_png || _png_info || _png_info_end)
  {
    png_destroy_read_struct(
      _png ? &_png : 0,
      _png_info ? &_png_info : 0,
      _png_info_end ? &_png_info_end : 0);
    _png = 0;
    _png_info = 0;
    _png_info_end = 0;
  }
  
  if (_f) { ::fclose(_f); _f = 0; }
}

namespace {

bool scanline(png_structp png, uint8_t * out)
{
  if (setjmp(png_jmpbuf(png))) return false;
  png_read_row(png, out, 0);
  return true;
}

bool end(png_structp png, png_infop png_info)
{
  if (setjmp(png_jmpbuf(png))) return false;
  png_read_end(png, png_info);
  return true;
}

}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  // Open.
  
  Info info;
  _open(_file.get(frame != -1 ? frame : _file.seq.start()), &info);
  
  // Read.
  
  Data * data = proxy ? &_tmp : &_image;
  data->set(info);
  
  for (int y = 0; y < info.size.y; ++y)
    if (! scanline(_png, data->data(0,  data->h() - 1 - y)))
      throw Error(name(), _png_error.msg);

  end(_png, _png_info_end);
  
  if (proxy)
  {
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}

