/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"

namespace djv_quicktime {
namespace io {

extern "C" {

DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load()
{
  quicktime_init();

  return new Load;
}

}

Load::Load() :
  _frame(0),
  _f(0)
{}

Load::~Load()
{
  _close();
}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_quicktime::name; }

const List<Str> & Load::extension_list() const
  { return djv_quicktime::extension_list; }

void Load::open(const File & in, Info * info, Tag * tag,
  djv_image::Time * time) throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _close();
  
  // Open.
  
  _file = in;
  
  _f = lqt_open_read(_file.get().c_str());
  if (! _f)
    throw Error(name(), djv_image::io::error_open);
  
  if (! quicktime_has_video(_f))
    throw Error(name(), error_video);

  if (! quicktime_supported_video(_f, 0))
    throw Error(name(), djv_image::io::error_unsupported);

  // Information.
  
  _info.name = in;
  
  _info.size = Vec2i(
    quicktime_video_width(_f, 0),
    quicktime_video_height(_f, 0));

  static int cmodel_list [] =
  {
    BC_RGB888,
    BC_BGR888,
    BC_RGB161616,
    BC_RGBA8888,
    BC_RGBA16161616,
    LQT_COLORMODEL_NONE
  };
  const int cmodel = lqt_get_best_colormodel(_f, 0, cmodel_list);
  switch (cmodel)
  {
    case BC_BGR888: _info.bgr = true;
    case BC_RGB888: _info.pixel = pixel::RGB_U8; break;
    case BC_RGB161616: _info.pixel = pixel::RGB_U16; break;
    case BC_RGBA8888: _info.pixel = pixel::RGBA_U8; break;
    case BC_RGBA16161616: _info.pixel = pixel::RGBA_U16; break;
    default: throw Error(name(), djv_image::io::error_unsupported);
  }
  lqt_set_cmodel(_f, 0, cmodel);
  
  int time_constant = 0;
  const int time_scale = lqt_video_time_scale(_f, 0);
  const int frame_duration = lqt_frame_duration(_f, 0, &time_constant);
  DJV_DBG_PRN("time scale = " << time_scale);
  DJV_DBG_PRN("frame duration = " << frame_duration);
  DJV_DBG_PRN("time constant = " << time_constant);
  
  _time = djv_image::Time(
    0, frame_duration ? (lqt_video_duration(_f, 0) / frame_duration - 1) : 0,
    djv_image::time::Speed(time_scale, frame_duration));
  DJV_DBG_PRN("time = " << _time);
  
  // Image tags.
  
  const List<Str> & label_tag = tag::label_tag();
  char * _tag = lqt_get_artist(_f);
  if (_tag) (*tag)[label_tag[tag::CREATOR]] = _tag;
  _tag = lqt_get_comment(_f);
  if (_tag) (*tag)[label_tag[tag::DESCRIPTION]] = _tag;
  _tag = quicktime_get_copyright(_f);
  if (_tag) (*tag)[label_tag[tag::COPYRIGHT]] = _tag;
  
  *info = _info;
  *time = _time;
}

void Load::open(const File & in, List<Info> * info, Tag * tag,
  djv_image::Time * time) throw (Error)
{
  djv_image::io::Load::open(in, info, tag, time);
}

void Load::_close()
{
  DJV_DBG_OBJ("Load::_close");
  
  if (_f) { quicktime_close(_f); _f = 0; }
}

namespace {

struct Rows
{
  Rows(Data * in) :
    p(0)
  {
    p = new unsigned char * [in->h() + 1];
    for (int y = 0; y < in->h(); ++y)
      p[y] = reinterpret_cast<pixel::U8_T *>(in->data(0, in->h() - 1 - y));
  }
    
  ~Rows()
  {
    if (p) delete [] p;
  }
    
  unsigned char ** p;
};

}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);
  
  if (-1 == frame) frame = 0;

  Data * data = proxy ? &_tmp : &_image;
  data->set(_info);

  // XXX
  
  if (frame != _frame + 1)
  {
    const uint64_t time = frame * _time.speed.duration;
    DJV_DBG_PRN("time = " << int(time));
    lqt_seek_video(_f, 0, time);
  }
  _frame = frame;
  
  lqt_decode_video(_f, Rows(data).p, 0);
  
  if (proxy)
  {
    Info info = _info;
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_tmp, &_image, proxy);
  }
  
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}

